(function() {
  var MetaStoreId, Mongo, MongoPersistTypes, MySQLPersistTypes, ObjectId, Redis, _, crypto, enhanceViews, enhancedViewMetaCache, entities, isDateOrTimeType, log, path, util, views;

  _ = require('lodash');

  crypto = require('crypto');

  ObjectId = require('mongodb').ObjectId;

  path = require('path');

  util = require('./util');

  log = require('./log');

  Mongo = require('./storage/Mongo');

  Redis = require('./storage/Redis');

  exports.DB = {
    mongo: 'mongodb',
    mysql: 'mysql',
    none: 'none'
  };

  exports.ObjectIdStringLength = 24;

  exports.FieldDataTypes = ["ObjectId", "String", "Password", "Boolean", "Int", "Float", "Date", "Time", "DateTime", "Image", "File", "Component", "Reference", "Object"];

  MongoPersistTypes = ["ObjectId", "String", "Boolean", "Number", "Date", 'Document'];

  MySQLPersistTypes = ["varchar", "char", "blob", "text", "int", "bit", "tinyint", "bigint", "decimal", "float", "double", "datetime", "date", "time", "timestamp"];

  exports.AllPersistTypes = MongoPersistTypes.concat(MySQLPersistTypes);

  exports.InputTypes = ["Text", "Password", "TextArea", "RichText", "Select", "Check", "Int", "Float", "CheckList", "Date", "Time", "DateTime", "File", "Image", "InlineComponent", "PopupComponent", "TabledComponent", "Reference"];

  exports.actions = {};

  isDateOrTimeType = function(fieldType) {
    return fieldType === "Date" || fieldType === "Time" || fieldType === "DateTime";
  };

  entities = null;

  views = null;

  enhancedViewMetaCache = null;

  MetaStoreId = new ObjectId().toString();

  Redis.subscribe('MetaChange', function*(metaStoreId) {
    if (metaStoreId !== MetaStoreId) {
      return;
    }
    log.system.info('ON MetaChange');
    return (yield* exports.gLoad());
  });

  exports.getEntityMeta = function(name) {
    return entities[name] || enhancedViewMetaCache[name];
  };

  exports.getEntities = function() {
    return entities;
  };

  exports.getViews = function() {
    return views;
  };

  exports.getMetaForFront = function() {
    return {
      entities: entities,
      views: enhancedViewMetaCache
    };
  };

  exports.gLoad = function*() {
    var SystemMeta, c, db, e, entitiesList, j, k, l, len, len1, ref, ref1, v, viewsList;
    SystemMeta = require('./SystemMeta');
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection('F_EntityMeta');
    entitiesList = (yield c.find({}).toArray());
    c = db.collection('F_EntityViewMeta');
    viewsList = (yield c.find({}).toArray());
    entities = {};
    for (j = 0, len = entitiesList.length; j < len; j++) {
      e = entitiesList[j];
      entities[e.name] = e;
    }
    views = {};
    for (l = 0, len1 = viewsList.length; l < len1; l++) {
      v = viewsList[l];
      views[v.name] = v;
    }
    ref = SystemMeta.SystemEntities;
    for (k in ref) {
      v = ref[k];
      entities[k] = v;
    }
    ref1 = SystemMeta.SystemViews;
    for (k in ref1) {
      v = ref1[k];
      views[k] = v;
    }
    enhanceViews();
    return log.system.info('Meta loaded');
  };

  exports.gSaveEntityMeta = function*(entityName, entityMeta) {
    var c, db;
    entityMeta._modifiedOn = new Date();
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection('F_EntityMeta');
    delete entityMeta._version;
    yield c.updateOne({
      name: entityName
    }, {
      $set: entityMeta,
      $inc: {
        _version: 1
      }
    }, {
      upsert: true
    });
    entities[entityName] = entityMeta;
    enhanceViews();
    return (yield* Redis.gPublish('MetaChange', MetaStoreId));
  };

  exports.gRemoveEntityMeta = function*(entityName) {
    var c, db;
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection('F_EntityMeta');
    yield c.removeOne({
      name: entityName
    });
    delete entities[entityName];
    return (yield* Redis.gPublish('MetaChange', MetaStoreId));
  };

  exports.gSaveViewMeta = function*(viewName, viewMeta) {
    var c, db;
    viewMeta._modifiedOn = new Date();
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection('F_EntityViewMeta');
    yield c.updateOne({
      name: viewName
    }, {
      $set: viewMeta,
      $inc: {
        _version: 1
      }
    }, {
      upsert: true
    });
    views[viewName] = viewMeta;
    enhanceViews();
    return (yield* Redis.gPublish('MetaChange', MetaStoreId));
  };

  exports.gRemoveViewMeta = function*(viewName) {
    var c, db;
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection('F_EntityViewMeta');
    yield c.removeOne({
      name: viewName
    });
    delete views[viewName];
    delete enhancedViewMetaCache[viewName];
    return (yield* Redis.gPublish('MetaChange', MetaStoreId));
  };

  enhanceViews = function() {
    var backEntityMeta, backFieldMeta, enhancedFieldMeta, enhancedViewMeta, fieldMeta, fieldName, k, ref, results, v, viewMeta, viewName;
    enhancedViewMetaCache = {};
    results = [];
    for (viewName in views) {
      viewMeta = views[viewName];
      backEntityMeta = entities[viewMeta.backEntity];
      enhancedViewMeta = _.clone(backEntityMeta);
      for (k in viewMeta) {
        v = viewMeta[k];
        if (v != null) {
          enhancedViewMeta[k] = v;
        }
      }
      enhancedViewMeta.fields = {};
      ref = viewMeta.fields;
      for (fieldName in ref) {
        fieldMeta = ref[fieldName];
        backFieldMeta = _.find(backEntityMeta.fields, function(f) {
          return f.name === fieldName;
        });
        enhancedFieldMeta = _.clone(backFieldMeta);
        for (k in fieldMeta) {
          v = fieldMeta[k];
          if (v != null) {
            enhancedFieldMeta[k] = v;
          }
        }
        enhancedViewMeta.fields[fieldName] = enhancedFieldMeta;
      }
      results.push(enhancedViewMetaCache[viewName] = enhancedViewMeta);
    }
    return results;
  };

  exports.parseEntity = function(entityInput, entityMeta) {
    var entityValue, fMeta, fName, fields, fv;
    if (entityInput == null) {
      return entityInput;
    }
    if (!_.isObject(entityInput)) {
      return void 0;
    }
    entityValue = {};
    fields = entityMeta.fields;
    for (fName in fields) {
      fMeta = fields[fName];
      fv = exports.parseFieldValue(entityInput[fName], fMeta);
      if (!(_.isUndefined(fv) || _.isNaN(fv))) {
        entityValue[fName] = fv;
      }
    }
    return entityValue;
  };

  exports.parseListQueryValue = function(criteria, entityMeta) {
    var fieldMeta, item, j, len, ref, results;
    if (criteria.relation) {
      ref = criteria.items;
      results = [];
      for (j = 0, len = ref.length; j < len; j++) {
        item = ref[j];
        results.push(exports.parseListQueryValue(item, entityMeta));
      }
      return results;
    } else if (criteria.field) {
      fieldMeta = entityMeta.fields[criteria.field];
      return criteria.value = exports.parseFieldValue(criteria.value, fieldMeta);
    }
  };

  exports.parseFieldValue = function(value, fieldMeta) {
    var i, idMeta, j, l, len, len1, len2, len3, len4, len5, len6, m, n, p, q, r, refEntityMeta, results, results1, results2, results3, results4, results5, results6;
    if (!fieldMeta) {
      return void 0;
    }
    if (value == null) {
      return value;
    }
    if (isDateOrTimeType(fieldMeta.type)) {
      if (_.isArray(value)) {
        results = [];
        for (j = 0, len = value.length; j < len; j++) {
          i = value[j];
          results.push(util.longToDate(i));
        }
        return results;
      } else {
        return util.longToDate(value);
      }
    } else if (fieldMeta.type === "ObjectId") {
      if (_.isArray(value)) {
        results1 = [];
        for (l = 0, len1 = value.length; l < len1; l++) {
          i = value[l];
          results1.push(Mongo.stringToObjectIdSilently(i));
        }
        return results1;
      } else {
        return Mongo.stringToObjectIdSilently(value);
      }
    } else if (fieldMeta.type === "Reference") {
      refEntityMeta = exports.getEntityMeta(fieldMeta.refEntity);
      if (refEntityMeta == null) {
        throw new Error("No ref entity [" + fieldMeta.refEntity + "]. Field " + fieldMeta.name);
      }
      idMeta = refEntityMeta.fields._id;
      return exports.parseFieldValue(value, idMeta);
    } else if (fieldMeta.type === "Boolean") {
      if (_.isArray(value)) {
        results2 = [];
        for (m = 0, len2 = value.length; m < len2; m++) {
          i = value[m];
          results2.push(util.stringToBoolean(i));
        }
        return results2;
      } else {
        return util.stringToBoolean(value);
      }
    } else if (fieldMeta.type === "Int") {
      if (_.isArray(value)) {
        results3 = [];
        for (n = 0, len3 = value.length; n < len3; n++) {
          i = value[n];
          results3.push(util.stringToInt(i));
        }
        return results3;
      } else {
        return util.stringToInt(value);
      }
    } else if (fieldMeta.type === "Float") {
      if (_.isArray(value)) {
        results4 = [];
        for (p = 0, len4 = value.length; p < len4; p++) {
          i = value[p];
          results4.push(util.stringToFloat(i));
        }
        return results4;
      } else {
        return util.stringToFloat(value);
      }
    } else if (fieldMeta.type === "Component") {
      refEntityMeta = exports.getEntityMeta(fieldMeta.refEntity);
      if (refEntityMeta == null) {
        throw new Error("No ref entity [" + fieldMeta.refEntity + "]. Field " + fieldMeta.name);
      }
      if (_.isArray(value)) {
        results5 = [];
        for (q = 0, len5 = value.length; q < len5; q++) {
          i = value[q];
          results5.push(exports.parseEntity(i, refEntityMeta));
        }
        return results5;
      } else {
        return exports.parseEntity(value, refEntityMeta);
      }
    } else if (fieldMeta.type === "Password") {
      if (!value) {
        return void 0;
      }
      if (_.isArray(value)) {
        results6 = [];
        for (r = 0, len6 = value.length; r < len6; r++) {
          i = value[r];
          results6.push(exports.hashPassword(i));
        }
        return results6;
      } else {
        return exports.hashPassword(value);
      }
    } else {
      if (value) {
        return value;
      } else {
        return null;
      }
    }
  };

  exports.parseId = function(id, entityMeta) {
    if (_.isString(entityMeta)) {
      entityMeta = exports.getEntityMeta(entityMeta);
    }
    return exports.parseFieldValue(id, entityMeta.fields._id);
  };

  exports.parseIds = function(ids, entityMeta) {
    var i, id, idMeta, j, len, list;
    if (ids == null) {
      return ids;
    }
    if (_.isString(entityMeta)) {
      entityMeta = exports.getEntityMeta(entityMeta);
    }
    idMeta = entityMeta.fields._id;
    list = [];
    for (j = 0, len = ids.length; j < len; j++) {
      id = ids[j];
      i = exports.parseFieldValue(id, idMeta);
      if (i != null) {
        list.push(i);
      }
    }
    return list;
  };

  exports.formatFieldToHttp = function(fieldValue, fieldMeta) {
    var i, j, l, len, len1, refEntityMeta, results, results1;
    if (fieldValue == null) {
      return fieldValue;
    }
    if (isDateOrTimeType(fieldMeta.type)) {
      if (fieldMeta.multiple) {
        results = [];
        for (j = 0, len = fieldValue.length; j < len; j++) {
          i = fieldValue[j];
          results.push(util.dateToLong(i));
        }
        return results;
      } else {
        return util.dateToLong(fieldValue);
      }
    } else if (fieldMeta.type === "Component") {
      refEntityMeta = exports.getEntityMeta(fieldMeta.refEntity);
      if (refEntityMeta == null) {
        throw new Error("No ref entity [" + fieldMeta.refEntity + "]. Field " + fieldMeta.name);
      }
      if (fieldMeta.multiple) {
        results1 = [];
        for (l = 0, len1 = fieldValue.length; l < len1; l++) {
          i = fieldValue[l];
          results1.push(exports.formatEntityToHttp(i, refEntityMeta));
        }
        return results1;
      } else {
        return exports.formatEntityToHttp(fieldValue, refEntityMeta);
      }
    } else if (fieldMeta.type === "Reference") {
      return fieldValue;
    } else if (fieldMeta.type === "Password") {
      return void 0;
    } else {
      return fieldValue;
    }
  };

  exports.formatEntityToHttp = function(entityValue, entityMeta) {
    var fName, fieldMeta, o, output, ref;
    if (entityValue == null) {
      return entityValue;
    }
    output = {};
    ref = entityMeta.fields;
    for (fName in ref) {
      fieldMeta = ref[fName];
      o = exports.formatFieldToHttp(entityValue[fName], fieldMeta);
      if (!_.isUndefined(o)) {
        output[fName] = o;
      }
    }
    return output;
  };

  exports.hashPassword = function(password) {
    if (password == null) {
      return password;
    }
    return crypto.createHash('md5').update(password + password).digest('hex');
  };

  exports.getCollectionName = function(entityMeta, repo) {
    switch (repo) {
      case 'trash':
        return entityMeta.tableName + "_trash";
      default:
        return entityMeta.tableName;
    }
  };

  exports.newObjectId = function() {
    return new ObjectId();
  };

  exports.imagePathsToImageObjects = function(paths, thumbnailFilled) {
    var j, len, o, results;
    if (!(paths != null ? paths.length : void 0)) {
      return paths;
    }
    results = [];
    for (j = 0, len = paths.length; j < len; j++) {
      path = paths[j];
      o = {
        path: path
      };
      if (thumbnailFilled) {
        o.thumbnail = path;
      }
      results.push(o);
    }
    return results;
  };

}).call(this);
