(function() {
  var Redis, _, gGet, gSet, keySeparator, log, test, util;

  Redis = require('../storage/Redis');

  _ = require('lodash');

  log = require('../log');

  util = require('../util');

  keySeparator = ":";

  gGet = function*(keys, alternative) {
    var key, ref;
    key = keys.join(keySeparator);
    return (yield ((ref = Redis.client.getAsync(key)) != null ? ref : alternative));
  };

  gSet = function*(keys, value) {
    var key;
    key = keys.join(keySeparator);
    return (yield Redis.client.setAsync(key, value));
  };

  exports.gGetString = gGet;

  exports.gSetString = gSet;

  exports.gGetCachedString = gGet;

  exports.gSetCachedString = gSet;

  exports.gGetObject = function*(keys, alternative) {
    var json, str;
    str = (yield* gGet(keys, alternative));
    json = str && JSON.parse(str);
    return util.typedJSONToJsObject(json);
  };

  exports.gSetObject = function*(keys, value) {
    var str;
    value = util.jsObjectToTypedJSON(value);
    str = value && JSON.stringify(value);
    return (yield* gSet(keys, str));
  };

  exports.gUnset = function*(keys, lastKeys) {
    var i, key, keys2, keysLength, lastKey, len;
    if (lastKeys != null ? lastKeys.length : void 0) {
      keys = _.clone(keys);
      keysLength = keys.length;
      keys2 = [];
      for (i = 0, len = lastKeys.length; i < len; i++) {
        lastKey = lastKeys[i];
        keys[keysLength] = lastKey;
        keys2.push(keys.join(keySeparator));
      }
      keys = keys2;
    } else {
      key = keys.join(keySeparator) + "*";
      keys = (yield Redis.client.keysAsync(key));
    }
    log.debug('unset redis keys', keys);
    if (keys.length) {
      return (yield Redis.client.delAsync(keys));
    }
  };

  exports.gClearAllCache = function*() {
    var keys;
    keys = (yield Redis.client.keysAsync("*"));
    if (keys.length) {
      return (yield Redis.client.delAsync(keys));
    }
  };

  test = function() {
    log.config({});
    Redis.init();
    return require('co')(function*() {
      var v;
      yield* exports.gSetObject(['aaa'], {
        a: 1,
        time: new Date(),
        aaa: [4, 5]
      });
      v = (yield* exports.gGetObject(['aaa']));
      console.log(v);
      return (yield* exports.gUnset(["a"], [1, 2, 3, 4]));
    })["catch"](function(e) {
      return log.system.error(e, 'test');
    });
  };

}).call(this);
