(function() {
  var UserService, authHandlers, error, gCheckAll, gCheckUserHasAction, gCheckUserHasEntityAction, log;

  log = require('../log');

  error = require('../error');

  UserService = require('../security/UserService');

  exports.gIdentifyUser = function*(next) {
    var e, trackId, user, userId, userToken;
    trackId = this.cookies.get('TID', {
      signed: true
    });
    this.state.trackId = trackId;
    userId = this.cookies.get('UserId', {
      signed: true
    });
    userToken = this.cookies.get('UserToken', {
      signed: true
    });
    if (userId && userToken) {
      try {
        user = (yield* UserService.gAuthToken(userId, userToken));
        if (user) {
          this.state.user = user;
        }
      } catch (error1) {
        e = error1;
        false;
      }
    }
    return (yield next);
  };

  exports.gControlAccess = function*(next) {
    var pass;
    pass = (yield* gCheckAll(this));
    if (!pass) {
      if (this.state.user != null) {
        throw new error.Error403();
      } else {
        throw new error.Error401();
      }
    }
    return (yield next);
  };

  gCheckAll = function*(httpCtx) {
    var gAuthHandler, ref, ri, route, state;
    route = httpCtx.route;
    state = httpCtx.state;
    ri = route.info;
    if (!(ri.auth || ri.action)) {
      return true;
    }
    if ((ref = state.user) != null ? ref.admin : void 0) {
      return true;
    }
    if (ri.action) {
      return (yield* gCheckUserHasAction(state.user, ri.action));
    } else if (ri.auth === true) {
      return state.user != null;
    } else {
      gAuthHandler = authHandlers[ri.auth];
      if (!gAuthHandler) {
        log.system.error('No auth handler for ' + ri.auth);
        return false;
      }
      return (yield* gAuthHandler(httpCtx));
    }
  };

  gCheckUserHasAction = function*(user, action) {
    var i, len, ref, ref1, ref2, ref3, role, roleId, roles;
    if (user == null) {
      return false;
    }
    if ((ref = user.acl) != null ? (ref1 = ref.action) != null ? ref1[action] : void 0 : void 0) {
      return true;
    }
    roles = user.roles;
    if (roles) {
      for (i = 0, len = roles.length; i < len; i++) {
        roleId = roles[i];
        role = (yield* UserService.gRoleById(roleId));
        if (role != null ? (ref2 = role.acl) != null ? (ref3 = ref2.action) != null ? ref3[action] : void 0 : void 0 : void 0) {
          return true;
        }
      }
    }
    return false;
  };

  authHandlers = {
    listEntity: function*(httpCtx) {
      return (yield* gCheckUserHasEntityAction(httpCtx.state.user, 'List', httpCtx.params.entityName));
    },
    getEntity: function*(httpCtx) {
      return (yield* gCheckUserHasEntityAction(httpCtx.state.user, 'Get', httpCtx.params.entityName));
    },
    createEntity: function*(httpCtx) {
      return (yield* gCheckUserHasEntityAction(httpCtx.state.user, 'Create', httpCtx.params.entityName));
    },
    updateOneEntity: function*(httpCtx) {
      return (yield* gCheckUserHasEntityAction(httpCtx.state.user, 'UpdateOne', httpCtx.params.entityName));
    },
    updateManyEntity: function*(httpCtx) {
      return (yield* gCheckUserHasEntityAction(httpCtx.state.user, 'UpdateMany', httpCtx.params.entityName));
    },
    removeEntity: function*(httpCtx) {
      return (yield* gCheckUserHasEntityAction(httpCtx.state.user, 'Remove', httpCtx.params.entityName));
    },
    recoverEntity: function*(httpCtx) {
      return (yield* gCheckUserHasEntityAction(httpCtx.state.user, 'Recover', httpCtx.params.entityName));
    }
  };

  gCheckUserHasEntityAction = function*(user, action, entityName) {
    var entityAcl, i, len, ref, ref1, ref2, ref3, ref4, ref5, role, roleId, roles;
    if (user != null) {
      entityAcl = (ref = user.acl) != null ? (ref1 = ref.entity) != null ? ref1[entityName] : void 0 : void 0;
      if (entityAcl && (entityAcl['*'] || entityAcl[action])) {
        return true;
      }
      roles = user.roles;
      if (roles) {
        for (i = 0, len = roles.length; i < len; i++) {
          roleId = roles[i];
          role = (yield* UserService.gRoleById(roleId));
          if (role) {
            entityAcl = (ref2 = role.acl) != null ? (ref3 = ref2.entity) != null ? ref3[entityName] : void 0 : void 0;
            if (entityAcl && (entityAcl['*'] || entityAcl[action])) {
              return true;
            }
          }
        }
      }
    } else {
      role = (yield* UserService.gGetAnonymousRole());
      if (role) {
        entityAcl = (ref4 = role.acl) != null ? (ref5 = ref4.entity) != null ? ref5[entityName] : void 0 : void 0;
        if (entityAcl && (entityAcl['*'] || entityAcl[action])) {
          return true;
        }
      }
    }
    return false;
  };

}).call(this);
