(function() {
  var Cache, EntityService, Interceptor, Meta, Mysql, _, checkAclField, error, log, util,
    slice = [].slice;

  _ = require('lodash');

  util = require('../util');

  error = require('../error');

  Meta = require('../Meta');

  log = require('../log');

  EntityService = require('../service/EntityService');

  Interceptor = require('./EntityInterceptor');

  Mysql = require('../storage/Mysql');

  Cache = require('../cache/Cache');

  exports.gCreateEntity = function*() {
    var entityMeta, entityName, fieldCount, gIntercept, instance, key, operator, r, ref, value;
    entityName = this.params.entityName;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!entityMeta) {
      throw new error.UserError('NoSuchEntity');
    }
    if (entityMeta.noCreate) {
      throw new error.UserError('CreateNotAllow');
    }
    instance = this.request.body;
    if (!instance) {
      throw new error.UserError("EmptyOperation");
    }
    instance = Meta.parseEntity(instance, entityMeta);
    exports.removeNoCreateFields(entityMeta, this.state.user, instance);
    fieldCount = 0;
    for (key in instance) {
      value = instance[key];
      if (_.isNull(value)) {
        delete instance[key];
      } else {
        fieldCount++;
      }
    }
    if (!fieldCount) {
      throw new error.UserError("EmptyOperation");
    }
    instance._createdBy = (ref = this.state.user) != null ? ref._id : void 0;
    gIntercept = Interceptor.getInterceptor(entityName, Interceptor.Actions.Create);
    operator = this.state.user;
    r = (yield* EntityService.gWithTransaction(entityMeta, function*(conn) {
      return (yield* gIntercept(conn, instance, operator, function*() {
        return (yield* EntityService.gCreate(conn, entityName, instance));
      }));
    }));
    return this.body = {
      id: r._id
    };
  };

  exports.gUpdateEntityById = function*() {
    var _id, criteria, entityMeta, entityName, gIntercept, instance, operator, ref;
    entityName = this.params.entityName;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!entityMeta) {
      throw new error.UserError('NoSuchEntity');
    }
    if (entityMeta.noEdit) {
      throw new error.UserError('EditNotAllow');
    }
    _id = Meta.parseId(this.params.id, entityMeta);
    if (_id == null) {
      return this.status = 404;
    }
    instance = this.request.body;
    criteria = {
      _id: _id,
      _version: instance._version
    };
    instance = Meta.parseEntity(instance, entityMeta);
    exports.removeNoEditFields(entityMeta, this.state.user, instance);
    instance._modifiedBy = (ref = this.state.user) != null ? ref._id : void 0;
    gIntercept = Interceptor.getInterceptor(entityName, Interceptor.Actions.Update);
    operator = this.state.user;
    yield* EntityService.gWithTransaction(entityMeta, function*(conn) {
      return (yield* gIntercept(conn, criteria, instance, operator, function*() {
        return (yield* EntityService.gUpdateOneByCriteria(conn, entityName, criteria, instance));
      }));
    });
    return this.status = 204;
  };

  exports.gUpdateEntityInBatch = function*() {
    var entityMeta, entityName, gIntercept, idVersions, iv, j, len, operator, patch, ref;
    entityName = this.params.entityName;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!entityMeta) {
      throw new error.UserError('NoSuchEntity');
    }
    if (entityMeta.noEdit) {
      throw new error.UserError('EditNotAllow');
    }
    patch = this.request.body;
    idVersions = patch.idVersions;
    if (!(idVersions.length > 0)) {
      throw new error.UserError('EmptyOperation');
    }
    delete patch.idVersions;
    for (j = 0, len = idVersions.length; j < len; j++) {
      iv = idVersions[j];
      iv.id = Meta.parseId(iv.id, entityMeta);
    }
    patch = Meta.parseEntity(patch, entityMeta);
    exports.removeNoEditFields(entityMeta, this.state.user, patch);
    patch._modifiedBy = (ref = this.state.user) != null ? ref._id : void 0;
    gIntercept = Interceptor.getInterceptor(entityName, Interceptor.Actions.Update);
    operator = this.state.user;
    yield* EntityService.gWithTransaction(entityMeta, function*(conn) {
      var criteria, k, len1, p, results;
      results = [];
      for (k = 0, len1 = idVersions.length; k < len1; k++) {
        p = idVersions[k];
        criteria = {
          _id: p.id,
          _version: p._version
        };
        results.push((yield* gIntercept(conn, criteria, patch, operator, function*() {
          return (yield* EntityService.gUpdateOneByCriteria(conn, entityName, criteria, patch));
        })));
      }
      return results;
    });
    return this.status = 204;
  };

  exports.gDeleteEntityInBatch = function*() {
    var criteria, entityMeta, entityName, gIntercept, ids, operator, ref;
    entityName = this.params.entityName;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!entityMeta) {
      throw new error.UserError('NoSuchEntity');
    }
    if (entityMeta.noDelete) {
      throw new error.UserError('DeleteNotAllow');
    }
    ids = (ref = this.query) != null ? ref._ids : void 0;
    if (!ids) {
      return this.status = 400;
    }
    ids = util.splitString(ids, ",");
    ids = Meta.parseIds(ids, entityMeta);
    if (!(ids.length > 0)) {
      throw new error.UserError('EmptyOperation');
    }
    criteria = {
      __type: 'relation',
      relation: 'and',
      items: [
        {
          field: '_id',
          operator: 'in',
          value: ids
        }
      ]
    };
    gIntercept = Interceptor.getInterceptor(entityName, Interceptor.Actions.Remove);
    operator = this.state.user;
    yield* EntityService.gWithTransaction(entityMeta, function*(conn) {
      return (yield* gIntercept(conn, criteria, operator, function*() {
        return (yield* EntityService.gRemoveManyByCriteria(conn, entityName, criteria));
      }));
    });
    return this.status = 204;
  };

  exports.gRecoverInBatch = function*() {
    var entityMeta, entityName, ids, ref;
    entityName = this.params.entityName;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!entityMeta) {
      throw new error.UserError('NoSuchEntity');
    }
    ids = (ref = this.request.body) != null ? ref.ids : void 0;
    if (!((ids != null ? ids.length : void 0) > 0)) {
      throw new error.UserError('EmptyOperation');
    }
    ids = Meta.parseIds(ids, entityMeta);
    if (!(ids.length > 0)) {
      throw new error.UserError('EmptyOperation');
    }
    yield* EntityService.gWithTransaction(entityMeta, function*(conn) {
      return (yield* EntityService.gRecoverMany(conn, entityName, ids));
    });
    return this.status = 204;
  };

  exports.gFindOneById = function*() {
    var _id, criteria, entity, entityMeta, entityName, gIntercept, operator;
    entityName = this.params.entityName;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!entityMeta) {
      throw new error.UserError('NoSuchEntity');
    }
    _id = Meta.parseId(this.params.id, entityMeta);
    if (_id == null) {
      return this.status = 404;
    }
    gIntercept = Interceptor.getInterceptor(entityName, Interceptor.Actions.Get);
    operator = this.state.user;
    criteria = {
      _id: _id
    };
    entity = (yield* EntityService.gWithoutTransaction(entityMeta, function*(conn) {
      return (yield* gIntercept(conn, criteria, operator, function*() {
        var ref;
        return (yield* EntityService.gFindOneByCriteria(conn, entityName, criteria, {
          repo: (ref = this.query) != null ? ref._repo : void 0
        }));
      }));
    }));
    if (entity) {
      exports.removeNotShownFields(entityMeta, this.state.user, entity);
      entity = Meta.formatEntityToHttp(entity, entityMeta);
      return this.body = entity;
    } else {
      return this.status = 404;
    }
  };

  exports.gList = function*() {
    var entityMeta, entityName, gIntercept, i, operator, page, query, r;
    entityName = this.params.entityName;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!entityMeta) {
      throw new error.UserError('NoSuchEntity');
    }
    query = exports.parseListQuery(entityMeta, this.query);
    gIntercept = Interceptor.getInterceptor(entityName, Interceptor.Actions.List);
    operator = this.state.user;
    r = (yield* EntityService.gWithoutTransaction(entityMeta, function*(conn) {
      return (yield* gIntercept(conn, query, operator, function*() {
        return (yield* EntityService.gList(conn, entityName, query));
      }));
    }));
    exports.removeNotShownFields.apply(exports, [entityMeta, this.state.user].concat(slice.call(r.page)));
    page = (function() {
      var j, len, ref, results;
      ref = r.page;
      results = [];
      for (j = 0, len = ref.length; j < len; j++) {
        i = ref[j];
        results.push(Meta.formatEntityToHttp(i, entityMeta));
      }
      return results;
    })();
    r.page = page;
    r.pageNo = query.pageNo;
    r.pageSize = query.pageSize;
    return this.body = r;
  };

  exports.parseListQuery = function(entityMeta, query) {
    var criteria, criteriaList, digest, e, fastFilter, field, fieldMeta, fieldName, fields, fs, includedFields, key, orList, pageNo, pageSize, ref, sort, sortBy, sortOrder, value;
    if (!query) {
      return {};
    }
    includedFields = util.splitString(query._includedFields, ",");
    digest = query._digest === 'true';
    if (digest) {
      includedFields = (function() {
        var j, len;
        if (entityMeta.digestFields) {
          fs = [];
          fields = entityMeta.digestFields.split(",");
          for (j = 0, len = fields.length; j < len; j++) {
            field = fields[j];
            fs = fs.concat(field.split('|'));
          }
          return fs;
        } else {
          return ["_id"];
        }
      })();
    }
    pageNo = util.stringToInt(query._pageNo, 1);
    pageSize = util.stringToInt(query._pageSize, digest && -1 || 20);
    if (pageSize > 200) {
      pageSize = 200;
    }
    fastFilter = query._filter;
    if (fastFilter) {
      orList = [];
      orList.push({
        field: "_id",
        operator: "==",
        value: fastFilter
      });
      ref = entityMeta.fields;
      for (fieldName in ref) {
        fieldMeta = ref[fieldName];
        if (fieldMeta.asFastFilter) {
          orList.push({
            field: fieldName,
            operator: "contain",
            value: fastFilter
          });
        }
      }
      criteria = {
        __type: 'relation',
        relation: "or",
        items: orList
      };
    } else {
      criteria = (function() {
        if (query._criteria) {
          try {
            return JSON.parse(query._criteria);
          } catch (error1) {
            e = error1;
            throw new error.UserError("BadQueryCriteria");
          }
        } else {
          criteriaList = [];
          for (key in query) {
            value = query[key];
            if (key in entityMeta.fields) {
              criteriaList.push({
                field: key,
                operator: "==",
                value: value
              });
            }
          }
          if (criteriaList.length) {
            return {
              __type: 'relation',
              relation: 'and',
              items: criteriaList
            };
          } else {
            return null;
          }
        }
      })();
      if (criteria) {
        Meta.parseListQueryValue(criteria, entityMeta);
        criteria.__type = 'relation';
      }
    }
    sort = (function() {
      var obj;
      if (query._sort) {
        try {
          return JSON.parse(query._sort);
        } catch (error1) {
          return null;
        }
      } else {
        sortBy = query._sortBy || '_createdOn';
        sortOrder = query._sortOrder === 'asc' ? 1 : -1;
        return (
          obj = {},
          obj["" + sortBy] = sortOrder,
          obj
        );
      }
    })();
    return {
      repo: query._repo,
      criteria: criteria,
      includedFields: includedFields,
      sort: sort,
      pageNo: pageNo,
      pageSize: pageSize
    };
  };

  exports.gSaveFilters = function*() {
    var criteria, entityMeta, includedFields, instance, lf, req;
    entityMeta = Meta.getEntityMeta('F_ListFilters');
    req = this.request.body;
    if (!req) {
      return this.status = 400;
    }
    instance = Meta.parseEntity(req, entityMeta);
    criteria = {
      name: instance.name,
      entityName: instance.entityName
    };
    includedFields = ['_id', '_version'];
    lf = (yield* EntityService.gFindOneByCriteria({}, 'F_ListFilters', criteria, {
      includedFields: includedFields
    }));
    if (lf) {
      yield* EntityService.gUpdateOneByCriteria({}, 'F_ListFilters', {
        _id: lf._id,
        _version: lf._version
      }, instance);
    } else {
      yield* EntityService.gCreate({}, 'F_ListFilters', instance);
    }
    return this.status = 204;
  };

  exports.gRemoveFilters = function*() {
    var query;
    query = this.query;
    if (!(query && query.name && query.entityName)) {
      return this.status = 400;
    }
    yield* EntityService.gRemoveManyByCriteria({}, 'F_ListFilters', {
      name: query.name,
      entityName: query.entityName
    });
    return this.status = 204;
  };

  checkAclField = function(user, entityName, fieldName, action) {
    var ref, ref1, ref2, ref3, ref4, ref5, ref6, ref7, ref8, role, roleName;
    if (!user) {
      return false;
    }
    if ((ref = user.acl) != null ? (ref1 = ref.field) != null ? (ref2 = ref1[entityName]) != null ? (ref3 = ref2[fieldName]) != null ? ref3[action] : void 0 : void 0 : void 0 : void 0) {
      return true;
    }
    if (user.roles) {
      ref4 = user.roles;
      for (roleName in ref4) {
        role = ref4[roleName];
        if ((ref5 = role.acl) != null ? (ref6 = ref5.field) != null ? (ref7 = ref6[entityName]) != null ? (ref8 = ref7[fieldName]) != null ? ref8[action] : void 0 : void 0 : void 0 : void 0) {
          return true;
        }
      }
    }
    return false;
  };

  exports.removeNotShownFields = function() {
    var e, entities, entityMeta, field, fieldMeta, fieldName, fields, j, len, removedFieldNames, results, user;
    entityMeta = arguments[0], user = arguments[1], entities = 3 <= arguments.length ? slice.call(arguments, 2) : [];
    fields = entityMeta.fields;
    removedFieldNames = [];
    for (fieldName in fields) {
      fieldMeta = fields[fieldName];
      if (fieldMeta.type === 'Password') {
        removedFieldNames.push(fieldName);
      } else if (fieldMeta.notShow) {
        if (!checkAclField(user, entityMeta.name, fieldName, 'show')) {
          removedFieldNames.push(fieldName);
        }
      }
    }
    if ((entities != null ? entities.length : void 0) && removedFieldNames.length) {
      results = [];
      for (j = 0, len = entities.length; j < len; j++) {
        e = entities[j];
        if (e != null) {
          results.push((function() {
            var k, len1, results1;
            results1 = [];
            for (k = 0, len1 = removedFieldNames.length; k < len1; k++) {
              field = removedFieldNames[k];
              delete e[field];
              results1.push(false);
            }
            return results1;
          })());
        }
      }
      return results;
    }
  };

  exports.removeNoCreateFields = function(entityMeta, user, entity) {
    var field, fieldMeta, fieldName, fields, j, len, removedFieldNames, results;
    fields = entityMeta.fields;
    removedFieldNames = [];
    for (fieldName in fields) {
      fieldMeta = fields[fieldName];
      if (fieldMeta.noCreate) {
        if (!checkAclField(user, entityMeta.name, fieldName, 'create')) {
          removedFieldNames.push(fieldName);
        }
      }
    }
    if (removedFieldNames.length) {
      results = [];
      for (j = 0, len = removedFieldNames.length; j < len; j++) {
        field = removedFieldNames[j];
        delete entity[field];
        results.push(false);
      }
      return results;
    }
  };

  exports.removeNoEditFields = function(entityMeta, user, entity) {
    var field, fieldMeta, fieldName, fields, j, len, removedFieldNames, results;
    fields = entityMeta.fields;
    removedFieldNames = [];
    for (fieldName in fields) {
      fieldMeta = fields[fieldName];
      if (fieldMeta.noEdit || fieldMeta.editReadonly) {
        if (!checkAclField(user, entityMeta.name, fieldName, 'edit')) {
          removedFieldNames.push(fieldName);
        }
      }
    }
    if (removedFieldNames.length) {
      results = [];
      for (j = 0, len = removedFieldNames.length; j < len; j++) {
        field = removedFieldNames[j];
        delete entity[field];
        results.push(false);
      }
      return results;
    }
  };

  exports.gViewMemoryCache = function*() {
    this.body = require('../cache/MemoryCache').cache;
  };

  exports.gClearAllCache = function*() {
    yield* Cache.gClearAllCache();
    return this.status = 204;
  };

}).call(this);
