(function() {
  var EntityService, Meta, bridges, error, getBridge;

  error = require('../error');

  Meta = require('../Meta');

  EntityService = require('../service/EntityService');

  bridges = {};

  getBridge = function(entityMeta) {
    return bridges[entityMeta.name];
  };

  exports.gCreate = function*(entityMeta, entityInput) {
    var bridge, ref;
    bridge = (ref = getBridge(entityMeta)) != null ? ref.gCreate : void 0;
    if (!bridge) {
      return entityMeta;
    }
    return (yield* bridge(entityMeta, entityInput));
  };

  exports.gUpdate = function*(entityMeta, entityInput, entityId) {
    var bridge, ref;
    bridge = (ref = getBridge(entityMeta)) != null ? ref.gUpdate : void 0;
    if (!bridge) {
      return entityMeta;
    }
    return (yield* bridge(entityMeta, entityInput, entityId));
  };

  exports.gUpdateInBatch = function*(entityMeta, entityInput, idVersions) {
    var bridge, ref;
    bridge = (ref = getBridge(entityMeta)) != null ? ref.gUpdateInBatch : void 0;
    if (!bridge) {
      return entityMeta;
    }
    return (yield* bridge(entityMeta, entityInput, idVersions));
  };

  exports.gDeleteInBatch = function*(entityMeta, ids) {
    var bridge, ref;
    bridge = (ref = getBridge(entityMeta)) != null ? ref.gDeleteInBatch : void 0;
    if (!bridge) {
      return entityMeta;
    }
    return (yield* bridge(entityMeta, ids));
  };

  exports.gRecoverInBatch = function*(entityMeta, ids) {
    var bridge, ref;
    bridge = (ref = getBridge(entityMeta)) != null ? ref.gRecoverInBatch : void 0;
    if (!bridge) {
      return entityMeta;
    }
    return (yield* bridge(entityMeta, ids));
  };

  exports.gAfterFindOne = function*(entityMeta, entity, operatorId) {
    var bridge, ref;
    bridge = (ref = getBridge(entityMeta)) != null ? ref.gAfterFindOne : void 0;
    if (!bridge) {
      return;
    }
    return (yield* bridge(entityMeta, entity, operatorId));
  };

  exports.gModifyListQuery = function*(entityMeta, query, operatorId) {
    var bridge, ref;
    bridge = (ref = getBridge(entityMeta)) != null ? ref.gModifyListQuery : void 0;
    if (!bridge) {
      return;
    }
    return (yield* bridge(entityMeta, query, operatorId));
  };

  bridges.StoreProduct = {
    onCreate: function*(entityMeta, entityInput) {
      var _createdBy, store;
      _createdBy = entityInput._createdBy;
      store = (yield* EntityService.gFindOneByCriteria(null, Meta.getEntityMeta('Store'), {
        staff: _createdBy
      }, {
        includeFields: ['_id']
      }));
      if (store == null) {
        throw new error.UserError('NotStaff', '无员工权限');
      }
      entityInput.store = store._id;
      entityMeta = Meta.getEntityMeta('GlobalProduct');
      return entityMeta;
    },
    onUpdate: function*(entityMeta, entityInput, entityId) {
      var _modifiedBy, globalProduct, storeOfModifier;
      _modifiedBy = entityInput._modifiedBy;
      delete entityInput.store;
      storeOfModifier = (yield* EntityService.gFindOneByCriteria(null, Meta.getEntityMeta('Store'), {
        staff: _modifiedBy
      }, {
        includeFields: ['_id']
      }));
      if (storeOfModifier == null) {
        throw new error.UserError('NotStaff', '无员工权限');
      }
      globalProduct = (yield* EntityService.gFindOneById(null, Meta.getEntityMeta('GlobalProduct'), entityId, {
        includeFields: ['store']
      }));
      if (globalProduct == null) {
        throw new error.UserError('NoSuchProduct', '无此商品');
      }
      if (globalProduct.store.toString() !== storeOfModifier._id.toString()) {
        throw new error.UserError('NotYourProduct', '您没有修改该商品的权限');
      }
      entityMeta = Meta.getEntityMeta('GlobalProduct');
      return {
        entityMeta: entityMeta,
        entityInput: entityInput
      };
    }
  };

}).call(this);
