(function() {
  var SecurityCodeService, UserService, checkPasswordFormat, config, error;

  error = require('../error');

  config = require('../config');

  UserService = require('../security/UserService');

  SecurityCodeService = require('../security/SecurityCodeService');

  checkPasswordFormat = function(password, format) {
    return format.test(password);
  };

  exports.checkPasswordFormat = checkPasswordFormat;

  exports.clearUserSessionCookies = function(ctx) {
    ctx.cookies.set('UserId', null, {
      signed: true,
      httpOnly: true
    });
    return ctx.cookies.set('UserToken', null, {
      signed: true,
      httpOnly: true
    });
  };

  exports.gPing = function*() {
    var i, len, ref, role, roleId, user, userToFront;
    if (this.state.user) {
      user = this.state.user;
      userToFront = {
        userId: user._id,
        admin: user.admin,
        acl: user.acl
      };
      userToFront.roles = {};
      if (user.roles) {
        ref = user.roles;
        for (i = 0, len = ref.length; i < len; i++) {
          roleId = ref[i];
          role = (yield* UserService.gRoleById(roleId));
          userToFront.roles[role.name] = role;
        }
      }
      return this.body = userToFront;
    } else {
      return this.status = 401;
    }
  };

  exports.gSignIn = function*() {
    var req, session;
    req = this.request.body;
    if (!(req.username && req.password)) {
      return this.status === 400;
    }
    session = (yield* UserService.gSignIn(req.username, req.password));
    this.body = {
      userId: session.userId
    };
    this.cookies.set('UserId', session.userId, {
      signed: true,
      httpOnly: true
    });
    return this.cookies.set('UserToken', session.userToken, {
      signed: true,
      httpOnly: true
    });
  };

  exports.toSignOutPage = function*() {
    yield* UserService.gSignOut(this.state.userId);
    exports.clearUserSessionCookies(this);
    return this.redirect('/sign-in');
  };

  exports.gSignOut = function*() {
    yield* UserService.gSignOut(this.state.userId);
    exports.clearUserSessionCookies(this);
    return this.status = 204;
  };

  exports.gChangePassword = function*() {
    var req;
    req = this.request.body;
    if (!checkPasswordFormat(req.newPassword, config.passwordFormat)) {
      throw new errors.UserError('BadPasswordFormat');
    }
    yield* UserService.gChangePassword(this.state.user._id, req.oldPassword, req.newPassword);
    exports.clearUserSessionCookies(this);
    return this.status = 204;
  };

  exports.gResetPassword = function*() {
    var req;
    req = this.request.body;
    if (!checkPasswordFormat(req.password, config.passwordFormat)) {
      throw new errors.UserError('BadPasswordFormat');
    }
    if (req.phone != null) {
      SecurityCodeService.check(req.phone, req.securityCode);
      yield* UserService.gResetPasswordByPhone(req.phone, req.password);
    } else if (req.email != null) {
      SecurityCodeService.check(req.email, req.securityCode);
      yield* UserService.gResetPasswordByEmail(req.email, req.password);
    } else {
      this.status = 400;
    }
    return this.status = 204;
  };

  exports.gChangePhone = function*() {
    var req;
    req = this.request.body;
    SecurityCodeService.check(req.phone, req.securityCode);
    yield* UserService.gChangePhone(this.state.user._id, req.phone);
    return this.status = 204;
  };

  exports.gChangeEmail = function*() {
    var req;
    req = this.request.body;
    SecurityCodeService.check(req.email, req.securityCode);
    yield* UserService.gChangeEmail(this.state.user._id, req.email);
    return this.status = 204;
  };

}).call(this);
