(function() {
  var Mongo, Mysql, Redis, WebServer, co, config, gStart, gStop, log, moment, onProcessTerm, stop, webStarted;

  moment = require('moment');

  moment.locale("zh-cn");

  co = require('co');

  log = require('./log');

  config = require('./config');

  Mongo = require('./storage/Mongo');

  Mysql = require('./storage/Mysql');

  Redis = require('./storage/Redis');

  WebServer = require('./web/WebServer');

  webStarted = false;

  gStart = function*(appConfig, addRouteRules) {
    var Meta, MongoIndex, MysqlIndex, RefactorMysqlTable, UserService, commonRouterRules, k, router, rrr, v;
    for (k in appConfig) {
      v = appConfig[k];
      config[k] = v;
    }
    log.config(config);
    console.log("\n\n\n\n\n");
    log.system.info('Starting FCMS...');
    Mongo.init();
    Mysql.init();
    if (config.cluster) {
      yield* Redis.gInit();
    }
    Meta = require('./Meta');
    yield* Meta.gLoad();
    MongoIndex = require('./storage/MongoIndex');
    yield* MongoIndex.gSyncWithMeta(Mongo.mongo);
    if (Mysql.mysql) {
      RefactorMysqlTable = require('./storage/RefactorMysqlTable');
      yield* RefactorMysqlTable.gSyncSchema(exports.mysql);
      MysqlIndex = require('./storage/MysqlIndex');
      yield* MysqlIndex.gSyncWithMeta(exports.mysql);
    }
    UserService = require('./security/UserService');
    UserService.init();
    yield* require('./init').gInit();
    router = require('./web/router');
    rrr = new router.RouteRuleRegisters(config.urlPrefix, config.errorCatcher);
    commonRouterRules = require('./web/commonRouterRules');
    commonRouterRules.addCommonRouteRules(rrr);
    if (typeof addRouteRules === "function") {
      addRouteRules(rrr);
    }
    log.system.info('Starting the web server...');
    yield* WebServer.gStart();
    webStarted = true;
    return log.system.info('Web server started!');
  };

  gStop = function*() {
    log.system.info('Disposing all other resources...');
    if (config.cluster) {
      yield* Redis.gDispose();
    }
    yield* Mongo.gDispose();
    yield* Mysql.gDispose();
    return log.system.info("ALL CLOSED!\n\n");
  };

  exports.start = function(appConfig, addRouteRules) {
    return co(function*() {
      return (yield* gStart(appConfig, addRouteRules));
    })["catch"](function(e) {
      log.system.error(e, 'Fail to start');
      return exports.stop();
    });
  };

  stop = function() {
    return co(gStop)["catch"](function(e) {
      return log.system.error(e, 'stop');
    });
  };

  onProcessTerm = function() {
    console.log("\n\n\n\n\n");
    log.system.info("The process terminating...");
    if (webStarted) {
      log.system.info("Closing web server firstly...");
      return WebServer.stop(stop);
    } else {
      return stop();
    }
  };

  process.on('SIGINT', onProcessTerm);

  process.on('SIGTERM', onProcessTerm);

}).call(this);
