(function() {
  var Cache, MailService, Promise, _generateSecurityCode, chance, error, gSendSecurityCodeToPhone, request, requestPost;

  chance = new require('chance')();

  Promise = require('bluebird');

  request = require('request');

  requestPost = Promise.promisify(request.post.bind(request));

  error = require('../error');

  Cache = require('../cache/Cache');

  MailService = require('../service/MailService');

  exports.check = function*(target, code) {
    var expectedCode;
    expectedCode = (yield* Cache.gGetString(['securityCodes', target]));
    if (!((expectedCode != null) && expectedCode.code === code)) {
      throw new error.UserError("SecurityCodeNotMatch");
    }
    if (new Date().getTime() - expectedCode.sendTime > 15 * 60 * 1000) {
      throw new error.UserError("SecurityCodeExpired");
    }
    return (yield* Cache.gUnset(['securityCodes'], [target]));
  };

  exports.gSendSecurityCodeToEmail = function*(toEmail, subject, purpose) {
    var code;
    code = _generateSecurityCode(toEmail);
    return (yield* MailService.gSendEmail(toEmail, subject, "您好，本次操作的验证码是 " + code + "。" + purpose + "。"));
  };

  gSendSecurityCodeToPhone = function*(phone, purpose) {
    var code, message, postData, r, ref, ref1;
    code = this._generateSecurityCode(phone);
    message = new Buffer("@1@=" + purpose + ",@2@=" + code, 'utf8').toString('utf8');
    postData = {
      method: "sendUtf8Msg",
      username: config.sms.username,
      password: config.sms.password,
      veryCode: config.sms.veryCode,
      mobile: phone,
      content: message,
      msgtype: 2,
      tempid: config.sms.template,
      code: "utf-8"
    };
    r = (yield requestPost(config.sms.url, {
      form: postData,
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8'
      }
    }));
    return ((ref = r.body) != null ? (ref1 = ref.match('<status>0</status>')) != null ? ref1.length : void 0 : void 0) > 0;
  };

  _generateSecurityCode = function*(address) {
    var code;
    code = chance.string({
      length: 6,
      pool: '0123456789'
    });
    yield* Cache.gSetString(['securityCodes', target], {
      code: code,
      sendTime: new Date().getTime()
    });
    return code;
  };

}).call(this);
