(function() {
  var Cache, EntityService, Meta, PermissionService, _gRemoveAllUserSessionOfUser, cacheKeyRoot, chance, config, error, log, util;

  chance = new require('chance')();

  error = require('../error');

  log = require('../log');

  Meta = require('../Meta');

  config = require('../config');

  util = require('../util');

  Cache = require('../cache/Cache');

  EntityService = require('../service/EntityService');

  PermissionService = require('./PermissionService');

  cacheKeyRoot = "user";

  exports.init = function() {
    var EntityServiceCache;
    EntityServiceCache = require('../service/EntityServiceCache');
    return EntityServiceCache.onUpdatedOrRemoved((function(_this) {
      return function*(ctx, entityMeta, ids) {
        if (entityMeta.name === 'F_User') {
          if (ids) {
            return (yield* Cache.gUnset([cacheKeyRoot, 'user'], ids));
          } else {
            return (yield* Cache.gUnset([cacheKeyRoot, 'user']));
          }
        } else if (entityMeta.name === 'F_UserRole') {
          yield* Cache.gUnset([cacheKeyRoot], ['anonymousRole']);
          if (ids) {
            return (yield* Cache.gUnset([cacheKeyRoot, 'role'], ids));
          } else {
            return (yield* Cache.gUnset([cacheKeyRoot, 'role']));
          }
        }
      };
    })(this));
  };

  exports.gUserById = function*(id) {
    var user;
    user = (yield* Cache.gGetObject([cacheKeyRoot, 'user', id]));
    if (user) {
      return user;
    }
    user = (yield* EntityService.gFindOneByCriteria({}, 'F_User', {
      _id: id
    }));
    if (user) {
      PermissionService.permissionArrayToMap(user.acl);
      yield* Cache.gSetObject([cacheKeyRoot, 'user', id], user);
    }
    return user;
  };

  exports.gRoleById = function*(id) {
    var role;
    role = (yield* Cache.gGetObject([cacheKeyRoot, 'role', id]));
    if (role) {
      return role;
    }
    role = (yield* EntityService.gFindOneByCriteria({}, 'F_UserRole', {
      _id: id
    }));
    if (role) {
      PermissionService.permissionArrayToMap(role.acl);
      yield* Cache.gSetObject([cacheKeyRoot, 'role', id], role);
    }
    return role;
  };

  exports.gRoleIdByName = function*(name) {
    var role;
    role = (yield* EntityService.gFindOneByCriteria({}, 'F_UserRole', {
      name: name
    }, {
      includeFields: ['_id']
    }));
    return role != null ? role._id : void 0;
  };

  exports.gAddRemoveRoleNameToUser = function*(userId, addRoles, removeRoles) {
    var addRoleIds, i, id, j, len, len1, name, removeRoleIds, roles, roles2, user;
    if (!(addRoles || removeRoles)) {
      return;
    }
    user = (yield* exports.gUserById(userId));
    roles = user.roles || [];
    if (addRoles) {
      addRoleIds = (yield* (function*() {
        var i, len, results;
        results = [];
        for (i = 0, len = addRoles.length; i < len; i++) {
          name = addRoles[i];
          results.push((yield* exports.gRoleIdByName(name)));
        }
        return results;
      })());
      for (i = 0, len = addRoleIds.length; i < len; i++) {
        id = addRoleIds[i];
        if (!util.inObjectIds(id, roles)) {
          roles.push(id);
        }
      }
    }
    if (removeRoles) {
      removeRoleIds = (yield* (function*() {
        var j, len1, results;
        results = [];
        for (j = 0, len1 = removeRoles.length; j < len1; j++) {
          name = removeRoles[j];
          results.push((yield* exports.gRoleIdByName(name)));
        }
        return results;
      })());
      roles2 = [];
      for (j = 0, len1 = roles.length; j < len1; j++) {
        id = roles[j];
        if (!util.inObjectIds(id, removeRoleIds)) {
          roles2.push(id);
        }
      }
      roles = roles2;
    }
    yield* EntityService.gUpdateOneByCriteria({}, 'F_User', {
      _id: userId
    }, {
      roles: roles
    });
    return (yield* Cache.gUnset([cacheKeyRoot, 'user'], [userId]));
  };

  exports.gGetAnonymousRole = function*() {
    var anonymousRole, role;
    anonymousRole = (yield* Cache.gGetObject([cacheKeyRoot, 'anonymousRole']));
    if (anonymousRole) {
      return anonymousRole;
    }
    role = (yield* EntityService.gFindOneByCriteria({}, 'F_UserRole', {
      name: 'anonymous'
    }));
    if (role) {
      PermissionService.permissionArrayToMap(role.acl);
      yield* Cache.gSetObject([cacheKeyRoot, 'anonymousRole'], role);
    }
    return role;
  };

  exports.gAuthToken = function*(userId, userToken) {
    var session;
    session = (yield* EntityService.gFindOneByCriteria({}, 'F_UserSession', {
      userId: userId
    }));
    if (!session) {
      return false;
    }
    if (session.userToken !== userToken) {
      log.debug('token not match', {
        userId: userId,
        userToken: userToken,
        sessionUserToken: session.userToken
      });
      return false;
    }
    if (session.expireAt < Date.now()) {
      log.debug('token expired', {
        userId: userId,
        expireAt: session.expireAt
      });
      return false;
    }
    return (yield* exports.gUserById(userId));
  };

  exports.gSignIn = function*(username, password) {
    var criteria, f, matchFields, session, user, usernameFields;
    if (!password) {
      throw new error.UserError("PasswordNotMatch");
    }
    usernameFields = config.usernameFields;
    if (!(usernameFields && usernameFields.length)) {
      usernameFields = ["username", "phone", "email"];
    }
    matchFields = (function() {
      var i, len, results;
      results = [];
      for (i = 0, len = usernameFields.length; i < len; i++) {
        f = usernameFields[i];
        results.push({
          field: f,
          operator: "==",
          value: username
        });
      }
      return results;
    })();
    criteria = {
      __type: 'relation',
      relation: 'or',
      items: matchFields
    };
    user = (yield* EntityService.gFindOneByCriteria({}, 'F_User', criteria));
    if (user == null) {
      throw new error.UserError("UserNotExisted");
    }
    if (user.disabled) {
      throw new error.UserError("UserDisabled");
    }
    if (Meta.hashPassword(password) !== user.password) {
      throw new error.UserError("PasswordNotMatch");
    }
    session = {};
    session.userId = user._id;
    session.userToken = chance.string({
      length: 20
    });
    session.expireAt = Date.now() + config.sessionExpireAtServer;
    yield* exports.gSignOut(user._id);
    yield* EntityService.gCreate({}, 'F_UserSession', session);
    return session;
  };

  exports.gSignOut = function*(userId) {
    var criteria;
    criteria = {
      userId: userId
    };
    return (yield* EntityService.gRemoveManyByCriteria({}, 'F_UserSession', criteria));
  };

  exports.gAddUser = function*(userInput) {
    var user;
    user = {
      _id: Meta.newObjectId().toString(),
      password: Meta.hashPassword(userInput.password)
    };
    if (userInput.username) {
      user.username = userInput.username;
    }
    if (userInput.phone) {
      user.phone = userInput.phone;
    }
    if (userInput.email) {
      user.email = userInput.email;
    }
    return (yield* EntityService.gCreate({}, 'F_User', user));
  };

  exports.gChangePhone = function*(userId, phone) {
    var user;
    user = (yield* EntityService.gFindOneByCriteria({}, 'F_User', {
      _id: userId
    }));
    if (user == null) {
      throw new error.UserError("UserNotExisted");
    }
    if (user.disabled) {
      throw new error.UserError("UserDisabled");
    }
    yield* EntityService.gUpdateOneByCriteria({}, 'F_User', {
      _id: userId,
      _version: user._version
    }, {
      phone: phone
    });
    return (yield* Cache.gUnset([cacheKeyRoot, 'user'], [userId]));
  };

  exports.gChangeEmail = function*(userId, email) {
    var user;
    user = (yield* EntityService.gFindOneByCriteria({}, 'F_User', {
      _id: userId
    }));
    if (user == null) {
      throw new error.UserError("UserNotExisted");
    }
    if (user.disabled) {
      throw new error.UserError("UserDisabled");
    }
    yield* EntityService.gUpdateOneByCriteria({}, 'F_User', {
      _id: userId,
      _version: user._version
    }, {
      email: email
    });
    return (yield* Cache.gUnset([cacheKeyRoot, 'user'], [userId]));
  };

  exports.gChangePassword = function*(userId, oldPassword, newPassword) {
    var update, user;
    user = (yield* EntityService.gFindOneByCriteria({}, 'F_User', {
      _id: userId
    }));
    if (user == null) {
      throw new error.UserError("UserNotExisted");
    }
    if (user.disabled) {
      throw new error.UserError("UserDisabled");
    }
    if (Meta.hashPassword(oldPassword) !== user.password) {
      throw new error.UserError("PasswordNotMatch");
    }
    update = {
      password: Meta.hashPassword(newPassword)
    };
    yield* EntityService.gUpdateOneByCriteria({}, 'F_User', {
      _id: userId,
      _version: user._version
    }, update);
    return (yield* _gRemoveAllUserSessionOfUser(userId));
  };

  exports.gResetPasswordByPhone = function*(phone, password) {
    var update, user;
    user = (yield* EntityService.gFindOneByCriteria({}, 'F_User', {
      phone: phone
    }));
    if (user == null) {
      throw new error.UserError("UserNotExisted");
    }
    if (user.disabled) {
      throw new error.UserError("UserDisabled");
    }
    update = {
      password: Meta.hashPassword(password)
    };
    yield* EntityService.gUpdateOneByCriteria({}, 'F_User', {
      _id: user._id,
      _version: user._version
    }, update);
    return (yield* _gRemoveAllUserSessionOfUser(user._id));
  };

  exports.gResetPasswordByEmail = function*(email, password) {
    var update, user;
    user = (yield* EntityService.gFindOneByCriteria({}, 'F_User', {
      email: email
    }));
    if (user == null) {
      throw new error.UserError("UserNotExisted");
    }
    if (user.disabled) {
      throw new error.UserError("UserDisabled");
    }
    update = {
      password: Meta.hashPassword(password)
    };
    yield* EntityService.gUpdateOneByCriteria({}, 'F_User', {
      _id: user._id,
      _version: user._version
    }, update);
    return (yield* _gRemoveAllUserSessionOfUser(user._id));
  };

  exports.checkUserHasRoleId = function(user, roleId) {
    var i, len, r, ref;
    roleId = roleId.toString();
    if (user.roles) {
      ref = user.roles;
      for (i = 0, len = ref.length; i < len; i++) {
        r = ref[i];
        if (r._id.toString() === roleId) {
          return true;
        }
      }
    }
    return false;
  };

  _gRemoveAllUserSessionOfUser = function*(userId) {
    return (yield* EntityService.gRemoveManyByCriteria({}, 'F_UserSession', {
      useId: userId
    }));
  };

}).call(this);
