(function() {
  var Promise, _, co, config, log, pRequest, request, test2;

  request = require('request');

  Promise = require('bluebird');

  pRequest = Promise.promisify(request);

  co = require('co');

  _ = require('lodash');

  config = require('../config');

  log = require('../log');

  exports.gConfigElasticSearch = function*(req) {
    var url;
    url = config.elasticSearchEndpoint + encodeURI(req.path);
    return (yield pRequest({
      url: url,
      method: req.method,
      json: req.body
    }));
  };

  exports.gIndexDocuments = function*(index, type, documents) {
    var doc, i, len, req, res, url;
    req = [];
    for (i = 0, len = documents.length; i < len; i++) {
      doc = documents[i];
      req.push(JSON.stringify({
        "index": {
          "_index": index,
          "_type": type,
          "_id": doc._id
        }
      }));
      doc = _.clone(doc);
      delete doc._id;
      req.push(JSON.stringify(doc));
    }
    url = config.elasticSearchEndpoint + "_bulk";
    res = (yield pRequest({
      url: url,
      method: 'POST',
      body: req.join("\n")
    }));
    if (res.statusCode !== 200) {
      throw new Error('gIndexDocuments fails ' + res.statusCode);
    }
  };

  exports.gIndexDocument = function*(index, type, id, doc) {
    var res, url;
    url = config.elasticSearchEndpoint + (index + "/" + type + "/" + id);
    res = (yield pRequest({
      url: url,
      method: 'PUT',
      json: doc
    }));
    if (!(res.statusCode === 200 || res.statusCode === 201)) {
      log.system.error(res.body, 'gIndexDocument');
      throw new Error('gIndexDocument fails ' + res.statusCode);
    }
  };

  exports.gRemoveDocument = function*(index, type, id) {
    var res, url;
    url = config.elasticSearchEndpoint + (index + "/" + type + "/" + id);
    res = (yield pRequest({
      url: url,
      method: 'DELETE'
    }));
    if (!(res.statusCode === 200 || res.statusCode === 204)) {
      throw new Error('gRemoveDocument fails ' + res.statusCode);
    } else {
      return log.system.info("gRemoveDocument " + index + "/" + type + " " + id);
    }
  };

  exports.removeDocumentAsync = function(index, type, id) {
    return co(function*() {
      return (yield* exports.gRemoveDocument(index, type, id));
    })["catch"](function(e) {
      return log.system.error(e, "removeDocumentAsync " + index + "/" + type);
    });
  };

  exports.gRemoveAllDocuments = function*(index) {
    var query, res, url;
    url = config.elasticSearchEndpoint + (index + "/_delete_by_query");
    query = {
      query: {
        match_all: {}
      }
    };
    res = (yield pRequest({
      url: url,
      method: 'POST',
      json: query
    }));
    log.debug('res', res.body);
    if (res.statusCode !== 200) {
      throw new Error('gRemoveAll fails ' + res.statusCode);
    }
  };

  test2 = function*() {
    return (yield* exports.gRemoveAllDocuments('game'));
  };

}).call(this);
