(function() {
  var EntityCache, Meta, MongoService, Mysql, MysqlService, _, error, util;

  _ = require('lodash');

  error = require('../error');

  util = require('../util');

  Meta = require('../Meta');

  MongoService = require('./EntityServiceMongo');

  MysqlService = require('./EntityServiceMysql');

  EntityCache = require('./EntityServiceCache');

  Mysql = require('../storage/Mysql');

  exports.gCreate = function*(conn, entityName, instance) {
    var entityMeta, id;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!_.size(instance)) {
      throw new error.UserError("CreateEmpty");
    }
    instance._version = 1;
    instance._createdOn = new Date();
    instance._modifiedOn = instance._createdOn;
    try {
      id = entityMeta.db === Meta.DB.mysql ? (yield* MysqlService.gCreate(conn, entityMeta, instance)) : entityMeta.db === Meta.DB.mongo ? (yield* MongoService.gCreate(entityMeta, instance)) : void 0;
      instance._id = id;
      return {
        _id: id
      };
    } finally {
      yield* EntityCache.gFireEntityCreated(conn, entityMeta);
    }
  };

  exports.gUpdateOneByCriteria = function*(conn, entityName, criteria, instance) {
    var entityMeta;
    entityMeta = Meta.getEntityMeta(entityName);
    delete instance._id;
    delete instance._version;
    delete instance._createdBy;
    delete instance._createdOn;
    if (!_.size(instance)) {
      return;
    }
    instance._modifiedOn = new Date();
    try {
      if (entityMeta.db === Meta.DB.mysql) {
        return (yield* MysqlService.gUpdateOneByCriteria(conn, entityMeta, criteria, instance));
      } else if (entityMeta.db === Meta.DB.mongo) {
        return (yield* MongoService.gUpdateOneByCriteria(entityMeta, criteria, instance));
      }
    } finally {
      yield* EntityCache.gFireEntityUpdated(conn, entityMeta, null);
    }
  };

  exports.gUpdateManyByCriteria = function*(conn, entityName, criteria, instance) {
    var entityMeta;
    entityMeta = Meta.getEntityMeta(entityName);
    delete instance._id;
    delete instance._version;
    delete instance._createdBy;
    delete instance._createdOn;
    if (!_.size(instance)) {
      return;
    }
    instance._modifiedOn = new Date();
    try {
      if (entityMeta.db === Meta.DB.mysql) {
        return (yield* MysqlService.gUpdateManyByCriteria(conn, entityMeta, criteria, instance));
      } else if (entityMeta.db === Meta.DB.mongo) {
        return (yield* MongoService.gUpdateManyByCriteria(entityMeta, criteria, instance));
      }
    } finally {
      yield* EntityCache.gFireEntityUpdated(conn, entityMeta, null);
    }
  };

  exports.gRemoveManyByCriteria = function*(conn, entityName, criteria) {
    var entityMeta;
    entityMeta = Meta.getEntityMeta(entityName);
    try {
      if (entityMeta.db === Meta.DB.mysql) {
        yield* MysqlService.gRemoveManyByCriteria(conn, entityMeta, criteria);
      } else if (entityMeta.db === Meta.DB.mongo) {
        yield* MongoService.gRemoveManyByCriteria(entityMeta, criteria);
      }
      return true;
    } finally {
      yield* EntityCache.gFireEntityRemoved(conn, entityMeta, null);
    }
  };

  exports.gRecoverMany = function*(conn, entityName, ids) {
    var entityMeta;
    entityMeta = Meta.getEntityMeta(entityName);
    try {
      if (entityMeta.db === Meta.DB.mysql) {
        return (yield* MysqlService.gRecoverMany(conn, entityMeta, ids));
      } else if (entityMeta.db === Meta.DB.mongo) {
        return (yield* MongoService.gRecoverMany(entityMeta, ids));
      }
    } finally {
      yield* EntityCache.gFireEntityCreated(conn, entityMeta);
    }
  };

  exports.gFindOneById = function*(conn, entityName, id, options) {
    var cacheId, criteria, entityMeta, ref;
    entityMeta = Meta.getEntityMeta(entityName);
    cacheId = id + "|" + (options != null ? options.repo : void 0) + "|" + (options != null ? (ref = options.includedFields) != null ? ref.join(",") : void 0 : void 0);
    criteria = {
      _id: id
    };
    return (yield* EntityCache.gWithCache(entityMeta, ["Id", cacheId], function*() {
      if (entityMeta.db === Meta.DB.mysql) {
        return (yield* MysqlService.gFindOneByCriteria(conn, entityMeta, criteria, options));
      } else if (entityMeta.db === Meta.DB.mongo) {
        return (yield* MongoService.gFindOneByCriteria(entityMeta, criteria, options));
      }
    }));
  };

  exports.gFindOneByCriteria = function*(conn, entityName, criteria, options) {
    var cacheId, entityMeta, ref;
    entityMeta = Meta.getEntityMeta(entityName);
    cacheId = "OneByCriteria|" + (options != null ? options.repo : void 0) + "|" + JSON.stringify(criteria) + "|" + (options != null ? (ref = options.includedFields) != null ? ref.join(",") : void 0 : void 0);
    return (yield* EntityCache.gWithCache(entityMeta, ["Other", cacheId], function*() {
      if (entityMeta.db === Meta.DB.mysql) {
        return (yield* MysqlService.gFindOneByCriteria(conn, entityMeta, criteria, options));
      } else if (entityMeta.db === Meta.DB.mongo) {
        return (yield* MongoService.gFindOneByCriteria(entityMeta, criteria, options));
      }
    }));
  };

  exports.gList = function*(conn, entityName, arg) {
    var cacheId, criteria, criteriaString, entityMeta, includedFields, includedFieldsString, pageNo, pageSize, repo, sort, sortString, withoutTotal;
    repo = arg.repo, criteria = arg.criteria, pageNo = arg.pageNo, pageSize = arg.pageSize, sort = arg.sort, includedFields = arg.includedFields, withoutTotal = arg.withoutTotal;
    entityMeta = Meta.getEntityMeta(entityName);
    if (!(pageNo >= 1)) {
      pageNo = 1;
    }
    sort = sort || {};
    criteria = criteria || {};
    criteriaString = JSON.stringify(criteria);
    sortString = util.objectToKeyValuePairString(sort);
    includedFieldsString = includedFields != null ? includedFields.join(',') : void 0;
    cacheId = "List|" + repo + "|" + pageNo + "|" + pageSize + "|" + criteriaString + "|" + sortString + "|" + includedFieldsString;
    return (yield* EntityCache.gWithCache(entityMeta, ["Other", cacheId], function*() {
      var query;
      query = {
        repo: repo,
        entityMeta: entityMeta,
        criteria: criteria,
        includedFields: includedFields,
        sort: sort,
        pageNo: pageNo,
        pageSize: pageSize,
        withoutTotal: withoutTotal
      };
      if (entityMeta.db === Meta.DB.mysql) {
        return (yield* MysqlService.gList(conn, query));
      } else if (entityMeta.db === Meta.DB.mongo) {
        return (yield* MongoService.gList(query));
      }
    }));
  };

  exports.gFindManyByCriteria = function*(conn, entityName, options) {
    var ref;
    options = options || {};
    options.pageSize = (ref = options.pageSize) != null ? ref : -1;
    options.withoutTotal = true;
    return (yield* exports.gList(conn, entityName, options));
  };

  exports.gFindManyByIds = function*(conn, entityName, ids, options) {
    options = options || {};
    options.criteria = {
      __type: 'relation',
      field: '_id',
      operator: 'in',
      value: ids
    };
    options.pageSize = -1;
    options.withoutTotal = true;
    return (yield* exports.gList(conn, entityName, options));
  };

  exports.gWithTransaction = function*(entityMeta, gWork) {
    if (entityMeta.db === Meta.DB.mysql) {
      return (yield* Mysql.mysql.gWithTransaction(function*(conn) {
        return (yield* gWork(conn));
      }));
    } else {
      return (yield* gWork());
    }
  };

  exports.gWithoutTransaction = function*(entityMeta, gWork) {
    if (entityMeta.db === Meta.DB.mysql) {
      return (yield* Mysql.mysql.gWithoutTransaction(function*(conn) {
        return (yield* gWork(conn));
      }));
    } else {
      return (yield* gWork());
    }
  };

}).call(this);
