(function() {
  var Cache, _, entityCreatedListeners, entityRemovedListeners, entityUpdatedListeners, gRemoveOneCacheByIds, log, sizeof, util;

  _ = require('lodash');

  sizeof = require('object-sizeof');

  util = require('../util');

  log = require('../log');

  Cache = require('../cache/Cache');

  entityCreatedListeners = [];

  entityUpdatedListeners = [];

  entityRemovedListeners = [];

  exports.gWithCache = function*(entityMeta, cacheId, gQuery) {
    var cacheItem, freshValue, keys, noServiceCache;
    noServiceCache = entityMeta.noServiceCache;
    if (noServiceCache) {
      return (yield* gQuery());
    } else {
      keys = _.concat(['Entity', entityMeta.name], cacheId);
      cacheItem = (yield* Cache.gGetObject(keys));
      if (cacheItem != null) {
        return _.cloneDeep(cacheItem);
      }
      freshValue = (yield* gQuery());
      if (freshValue == null) {
        return freshValue;
      }
      yield* Cache.gSetObject(keys, freshValue);
      return _.cloneDeep(freshValue);
    }
  };

  exports.onEntityCreated = function(listener) {
    return entityCreatedListeners.push(listener);
  };

  exports.onEntityUpdated = function(listener) {
    return entityUpdatedListeners.push(listener);
  };

  exports.onEntityRemoved = function(listener) {
    return entityRemovedListeners.push(listener);
  };

  exports.onUpdatedOrRemoved = function(listener) {
    entityUpdatedListeners.push(listener);
    return entityRemovedListeners.push(listener);
  };

  exports.gFireEntityCreated = function*(ctx, entityMeta) {
    var e, i, l, len, results;
    yield* Cache.gUnset(['Entity', entityMeta.name, 'Other']);
    results = [];
    for (i = 0, len = entityCreatedListeners.length; i < len; i++) {
      l = entityCreatedListeners[i];
      try {
        if (util.isGenerator(l)) {
          results.push((yield* l(ctx, entityMeta)));
        } else {
          results.push(l(ctx, entityMeta));
        }
      } catch (error) {
        e = error;
        log.system.error(e, "fireEntityCreated");
        throw e;
      }
    }
    return results;
  };

  exports.gFireEntityUpdated = function*(ctx, entityMeta, ids) {
    var e, i, l, len, results;
    yield* Cache.gUnset(['Entity', entityMeta.name, 'Other']);
    yield* gRemoveOneCacheByIds(entityMeta, ids);
    results = [];
    for (i = 0, len = entityUpdatedListeners.length; i < len; i++) {
      l = entityUpdatedListeners[i];
      try {
        if (util.isGenerator(l)) {
          results.push((yield* l(ctx, entityMeta, ids)));
        } else {
          results.push(l(ctx, entityMeta, ids));
        }
      } catch (error) {
        e = error;
        log.system.error(e, "onEntityUpdated");
        throw e;
      }
    }
    return results;
  };

  exports.gFireEntityRemoved = function*(ctx, entityMeta, ids) {
    var e, i, l, len, results;
    yield* Cache.gUnset(['Entity', entityMeta.name, 'Other']);
    yield* gRemoveOneCacheByIds(entityMeta, ids);
    results = [];
    for (i = 0, len = entityRemovedListeners.length; i < len; i++) {
      l = entityRemovedListeners[i];
      try {
        if (util.isGenerator(l)) {
          results.push((yield* l(ctx, entityMeta, ids)));
        } else {
          results.push(l(ctx, entityMeta, ids));
        }
      } catch (error) {
        e = error;
        log.system.error(e, "onEntityRemoved");
        throw e;
      }
    }
    return results;
  };

  gRemoveOneCacheByIds = function*(entityMeta, ids) {
    var i, id, len, results;
    if (ids) {
      results = [];
      for (i = 0, len = ids.length; i < len; i++) {
        id = ids[i];
        results.push((yield* Cache.gUnset(['Entity', entityMeta.name, 'Id', id])));
      }
      return results;
    } else {
      return (yield* Cache.gUnset(['Entity', entityMeta.name, 'Id']));
    }
  };

}).call(this);
