(function() {
  var Meta, Mongo, _, _gRemoveManyCompletely, _gRemoveManyToTrash, _objectToMongoUpdate, _toDupKeyError, error, log, util;

  _ = require('lodash');

  error = require('../error');

  Meta = require('../Meta');

  log = require('../log');

  util = require('../util');

  Mongo = require('../storage/Mongo');

  exports.gCreate = function*(entityMeta, instance) {
    var c, code, db, e, message, ref, res;
    if (entityMeta.fields._id.persistType === 'String' && (instance._id == null)) {
      instance._id = Meta.newObjectId().toString();
    }
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection(entityMeta.tableName);
    try {
      res = (yield c.insertOne(instance));
      return Mongo.getInsertedIdObject(res);
    } catch (error1) {
      e = error1;
      if (!Mongo.isIndexConflictError(e)) {
        throw e;
      }
      ref = _toDupKeyError(e, entityMeta), code = ref.code, message = ref.message;
      throw new error.UniqueConflictError(code, message);
    }
  };

  exports.gUpdateManyByCriteria = function*(entityMeta, criteria, instance) {
    var c, code, db, e, message, nativeCriteria, r, ref, res, update;
    update = _objectToMongoUpdate(instance);
    if (!update) {
      return 0;
    }
    nativeCriteria = Mongo.toMongoCriteria(criteria);
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection(entityMeta.tableName);
    try {
      res = (yield c.updateMany(nativeCriteria, update));
      r = Mongo.getUpdateResult(res);
      return r.modifiedCount;
    } catch (error1) {
      e = error1;
      if (!Mongo.isIndexConflictError(e)) {
        throw e;
      }
      ref = _toDupKeyError(e, entityMeta), code = ref.code, message = ref.message;
      throw new error.UniqueConflictError(code, message);
    }
  };

  exports.gUpdateOneByCriteria = function*(entityMeta, criteria, instance) {
    var c, code, db, e, message, nativeCriteria, r, ref, res, update;
    update = _objectToMongoUpdate(instance);
    if (!update) {
      return 0;
    }
    nativeCriteria = Mongo.toMongoCriteria(criteria);
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection(entityMeta.tableName);
    try {
      res = (yield c.updateOne(nativeCriteria, update));
      r = Mongo.getUpdateResult(res);
      if (r.modifiedCount !== 1) {
        throw new error.UserError('ConcurrentUpdate');
      }
    } catch (error1) {
      e = error1;
      if (!Mongo.isIndexConflictError(e)) {
        throw e;
      }
      ref = _toDupKeyError(e, entityMeta), code = ref.code, message = ref.message;
      throw new error.UniqueConflictError(code, message);
    }
  };

  exports.gRemoveManyByCriteria = function*(entityMeta, criteria) {
    var nativeCriteria;
    nativeCriteria = Mongo.toMongoCriteria(criteria);
    if (entityMeta.removeMode === 'toTrash') {
      return (yield* _gRemoveManyToTrash(entityMeta, nativeCriteria));
    } else {
      return (yield* _gRemoveManyCompletely(entityMeta, nativeCriteria));
    }
  };


  /*
  软删除有几种方式：放在单独的表中，放在原来的表中+使用标记字段。
  放在单独的表中，在撤销删除后，有id重复的风险：例如删除id为1的实体，其后又产生了id为1的实体，则把删除的实体找回后就主键冲突了
  好在目前采用ObjectId的方式不会导致该问题。
  放在原表加标记字段的方式，使得所有的查询都要记得查询删除标记为false的实体，并影响索引的构建，麻烦
   */

  _gRemoveManyToTrash = function*(entityMeta, criteria) {
    var db, entity, formalCollection, j, len, list, trashCollection, trashTable;
    trashTable = Meta.getCollectionName(entityMeta, "trash");
    db = (yield* Mongo.mongo.gDatabase());
    formalCollection = db.collection(entityMeta.tableName);
    trashCollection = db.collection(trashTable);
    list = (yield formalCollection.find(criteria).toArray());
    for (j = 0, len = list.length; j < len; j++) {
      entity = list[j];
      entity._modifiedOn = new Date();
      entity._version++;
    }
    yield trashCollection.insertMany(list);
    return (yield formalCollection.deleteMany(criteria));
  };

  _gRemoveManyCompletely = function*(entityMeta, criteria) {
    var c, db;
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection(entityMeta.tableName);
    return (yield c.deleteMany(criteria));
  };

  exports.gRecoverMany = function*(entityMeta, ids) {
    var code, db, e, entity, formalCollection, j, len, list, message, ref, trashCollection, trashTable;
    trashTable = Meta.getCollectionName(entityMeta, "trash");
    db = (yield* Mongo.mongo.gDatabase());
    formalCollection = db.collection(entityMeta.tableName);
    trashCollection = db.collection(trashTable);
    list = (yield trashCollection.find({
      _id: {
        $in: ids
      }
    }).toArray());
    for (j = 0, len = list.length; j < len; j++) {
      entity = list[j];
      entity._modifiedOn = new Date();
      entity._version++;
    }
    try {
      yield formalCollection.insertMany(list);
    } catch (error1) {
      e = error1;
      if (!Mongo.isIndexConflictError(e)) {
        throw e;
      }
      ref = _toDupKeyError(e, entityMeta), code = ref.code, message = ref.message;
      throw new error.UniqueConflictError(code, message);
    }
    return (yield trashCollection.deleteMany({
      _id: {
        $in: ids
      }
    }));
  };

  exports.gFindOneByCriteria = function*(entityMeta, criteria, o) {
    var c, collectionName, db, nativeCriteria, projection;
    collectionName = Meta.getCollectionName(entityMeta, o != null ? o.repo : void 0);
    nativeCriteria = Mongo.toMongoCriteria(criteria);
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection(collectionName);
    projection = util.arrayToTrueObject(o != null ? o.includedFields : void 0) || {};
    return (yield c.findOne(nativeCriteria, projection));
  };

  exports.gList = function*(arg) {
    var c, collectionName, criteria, cursor, db, entityMeta, includedFields, nativeCriteria, page, pageNo, pageSize, repo, sort, total, withoutTotal;
    entityMeta = arg.entityMeta, criteria = arg.criteria, sort = arg.sort, repo = arg.repo, includedFields = arg.includedFields, pageNo = arg.pageNo, pageSize = arg.pageSize, withoutTotal = arg.withoutTotal;
    collectionName = Meta.getCollectionName(entityMeta, repo);
    nativeCriteria = Mongo.toMongoCriteria(criteria);
    includedFields = util.arrayToTrueObject(includedFields) || {};
    db = (yield* Mongo.mongo.gDatabase());
    c = db.collection(collectionName);
    if (!withoutTotal) {
      total = (yield c.count(nativeCriteria));
    }
    cursor = c.find(nativeCriteria, includedFields).sort(sort);
    if (pageSize > 0) {
      cursor.skip((pageNo - 1) * pageSize).limit(pageSize);
    }
    page = (yield cursor.toArray());
    if (withoutTotal) {
      return page;
    } else {
      return {
        total: total,
        page: page
      };
    }
  };

  _toDupKeyError = function(e, entityMeta) {
    var indexConfig, indexName, matches, message;
    matches = e.message.match(/index:\s(.+)\$(.+) dup key: (.+)/);
    if (matches) {
      indexName = matches[2];
      log.debug('toDupKeyError, indexName=' + indexName);
      indexConfig = _.find(entityMeta.mongoIndexes, function(i) {
        return entityMeta.tableName + "_" + i.name === indexName;
      });
      if (!indexConfig) {
        log.system.warn('No index config for ' + indexName);
      }
      message = (indexConfig != null ? indexConfig.errorMessage : void 0) || ("值重复：" + indexName);
      return {
        code: "DupKey",
        message: message,
        key: indexName
      };
    } else {
      return {
        code: "DupKey",
        message: e.message,
        key: null
      };
    }
  };

  _objectToMongoUpdate = function(object) {
    var key, set, unset, update, value;
    if (!_.size(object)) {
      return null;
    }
    delete object._version;
    delete object._id;
    set = {};
    unset = {};
    for (key in object) {
      value = object[key];
      if (value != null) {
        set[key] = value;
      } else {
        unset[key] = "";
      }
    }
    update = {
      $inc: {
        _version: 1
      }
    };
    if (_.size(set)) {
      update.$set = set;
    }
    if (_.size(unset)) {
      update.$unset = unset;
    }
    return update;
  };

}).call(this);
