(function() {
  var Meta, Mysql, _, _gRemoveManyCompletely, _gRemoveManyToTrash, _toDupKeyError, error;

  _ = require('lodash');

  error = require('../error');

  Meta = require('../Meta');

  Mysql = require('../storage/Mysql');

  exports.gCreate = function*(conn, entityMeta, instance) {
    var code, e, id, message, ref;
    if (!_.size(instance)) {
      return null;
    }
    id = Meta.newObjectId().toString();
    instance._id = id;
    try {
      yield* conn.gInsertOne(entityMeta.tableName, instance);
      return id;
    } catch (error1) {
      e = error1;
      if (!Mysql.isIndexConflictError(e)) {
        throw e;
      }
      ref = _toDupKeyError(e, entityMeta), code = ref.code, message = ref.message;
      throw new error.UniqueConflictError(code, message);
    }
  };

  exports.gUpdateManyByCriteria = function*(conn, entityMeta, criteria, instance) {
    var code, e, message, ref;
    if (!_.size(instance)) {
      return 0;
    }
    try {
      return (yield* conn.gUpdateByCriteria(entityMeta.tableName, criteria, instance));
    } catch (error1) {
      e = error1;
      if (!Mysql.isIndexConflictError(e)) {
        throw e;
      }
      ref = _toDupKeyError(e, entityMeta), code = ref.code, message = ref.message;
      throw new error.UniqueConflictError(code, message);
    }
  };

  exports.gUpdateOneByCriteria = function*(conn, entityMeta, criteria, instance) {
    var code, e, message, r, ref;
    if (!_.size(instance)) {
      return 0;
    }
    try {
      if (instance._version >= 0) {
        instance._version = instance._version + 1;
      }
      r = (yield* conn.gUpdateByCriteria(entityMeta.tableName, criteria, instance));
      if ((r != null ? r.changedRows : void 0) !== 1) {
        throw new error.UserError('ConcurrentUpdate');
      }
    } catch (error1) {
      e = error1;
      if (!Mysql.isIndexConflictError(e)) {
        throw e;
      }
      ref = _toDupKeyError(e, entityMeta), code = ref.code, message = ref.message;
      throw new error.UniqueConflictError(code, message);
    }
  };

  exports.gRemoveManyByCriteria = function*(conn, entityMeta, criteria) {
    if (!(typeof ids !== "undefined" && ids !== null ? ids.length : void 0)) {
      return;
    }
    if (entityMeta.removeMode === 'toTrash') {
      return (yield* _gRemoveManyToTrash(conn, entityMeta, criteria));
    } else {
      return (yield* _gRemoveManyCompletely(conn, entityMeta, criteria));
    }
  };

  exports.gRecoverMany = function*(conn, entityMeta, ids) {
    var entity, j, len, list, trashTable;
    if (!(ids != null ? ids.length : void 0)) {
      return;
    }
    trashTable = Meta.getCollectionName(entityMeta, "trash");
    list = (yield* conn.gListByIds(trashTable, ids));
    for (j = 0, len = list.length; j < len; j++) {
      entity = list[j];
      entity._modifiedOn = new Date();
      entity._version++;
    }
    yield* conn.gInsertMany(entityMeta.tableName, list);
    return (yield* conn.gDeleteManyByIds(trashTable, ids));
  };

  _gRemoveManyToTrash = function*(conn, entityMeta, criteria) {
    var entity, j, len, list, trashTable;
    if (!(typeof ids !== "undefined" && ids !== null ? ids.length : void 0)) {
      return;
    }
    trashTable = Meta.getCollectionName(entityMeta, "trash");
    list = (yield* conn.gFind({
      table: entityMeta.tableName,
      criteria: criteria,
      pageSize: -1,
      paging: false
    }));
    for (j = 0, len = list.length; j < len; j++) {
      entity = list[j];
      entity._modifiedOn = new Date();
      entity._version++;
    }
    if (list != null ? list.length : void 0) {
      yield* conn.gInsertMany(trashTable, list);
    }
    return (yield* conn.gDeleteManyByCriteria(entityMeta.tableName, criteria));
  };

  _gRemoveManyCompletely = function*(conn, entityMeta, criteria) {
    if (!(typeof ids !== "undefined" && ids !== null ? ids.length : void 0)) {
      return;
    }
    return (yield* conn.gDeleteManyByCriteria(entityMeta.tableName, criteria));
  };

  exports.gFindOneByCriteria = function*(conn, entityMeta, criteria, o) {
    var list, table;
    table = Meta.getCollectionName(entityMeta, o != null ? o.repo : void 0);
    list = (yield* conn.gFind({
      table: table,
      criteria: criteria,
      includedFields: o != null ? o.includedFields : void 0,
      pageNo: 1,
      pageSize: 1
    }));
    return list.length && list[0] || null;
  };

  exports.gList = function*(conn, arg) {
    var criteria, entityMeta, includedFields, pageNo, pageSize, repo, sort, table, withoutTotal;
    repo = arg.repo, entityMeta = arg.entityMeta, criteria = arg.criteria, includedFields = arg.includedFields, sort = arg.sort, pageNo = arg.pageNo, pageSize = arg.pageSize, withoutTotal = arg.withoutTotal;
    table = Meta.getCollectionName(entityMeta, repo);
    return (yield* conn.gFind({
      table: table,
      criteria: criteria,
      includedFields: includedFields,
      sort: sort,
      pageNo: pageNo,
      pageSize: pageSize,
      paging: withoutTotal
    }));
  };

  _toDupKeyError = function(e, entityMeta) {
    var indexConfig, key, matches, message, specifiedKey;
    matches = e.message.match(/Duplicate entry '(.*)' for key '(.+)'$/);
    if (matches) {
      key = matches[2];
      specifiedKey = entityMeta.tableName + "_" + key;
      indexConfig = _.find(entityMeta.mysqlIndexes, function(i) {
        return i.name === specifiedKey;
      });
      message = (indexConfig != null ? indexConfig.errorMessage : void 0) || ("值重复：" + key);
      return {
        code: "DupKey",
        message: message,
        key: key
      };
    } else {
      return {
        code: "DupKey",
        message: e.message,
        key: null
      };
    }
  };

}).call(this);
