(function() {
  var MongoClient, MongoStore, ObjectId, config, log, mongodb, toMongoCriteria;

  mongodb = require('mongodb');

  ObjectId = mongodb.ObjectId;

  MongoClient = mongodb.MongoClient;

  log = require('../log');

  config = require('../config');

  MongoStore = (function() {
    function MongoStore(name, url) {
      this.name = name;
      this.url = url;
    }

    MongoStore.prototype.gDatabase = function*() {
      var db;
      if (this.db) {
        return this.db;
      }
      db = (yield MongoClient.connect(this.url));
      db.on('close', (function(_this) {
        return function() {
          _this.db = null;
          return log.system.info("MongoDB [" + _this.name + "] closed");
        };
      })(this));
      db.on('error', (function(_this) {
        return function(e) {
          _this.db = null;
          return log.system.error(e, "MongoDB [" + _this.name + "] error");
        };
      })(this));
      db.on('reconnect', (function(_this) {
        return function() {
          return log.system.info("Mongo DB [" + _this.name + "] reconnect");
        };
      })(this));
      this.db = db;
      return db;
    };

    MongoStore.prototype.gDispose = function*() {
      var e;
      log.system.info("Closing mongodb [" + this.name + "]...");
      if (!this.db) {
        return;
      }
      try {
        return (yield this.db.close());
      } catch (error) {
        e = error;
        return log.system.error(e, "dispose mongodb [" + this.name + "]");
      }
    };

    return MongoStore;

  })();

  exports.init = function() {
    var ref;
    if (!((ref = config.mongo) != null ? ref.url : void 0)) {
      throw new Error('Mongo config is required now');
    }
    return exports.mongo = new MongoStore('main', config.mongo.url);
  };

  exports.gDispose = function*() {
    if (exports.mongo) {
      return (yield* exports.mongo.gDispose());
    }
  };

  exports.getInsertedIdObject = function(r) {
    return (r != null ? r.insertedId : void 0) || null;
  };

  exports.getUpdateResult = function(r) {
    return {
      matchedCount: r.matchedCount,
      modifiedCount: r.modifiedCount
    };
  };

  exports.isIndexConflictError = function(e) {
    return e.code === 11000;
  };

  exports.stringToObjectId = function(string) {
    if (!string) {
      return null;
    }
    if (string instanceof ObjectId) {
      return string;
    } else {
      return new ObjectId(string);
    }
  };

  exports.stringToObjectIdSilently = function(string) {
    if (string instanceof ObjectId) {
      return string;
    }
    if (string == null) {
      return string;
    }
    try {
      return new ObjectId(string);
    } catch (error) {
      return void 0;
    }
  };

  exports.stringArrayToObjectIdArraySilently = function(strings) {
    var i, id, ids, len, s;
    if (strings == null) {
      return [];
    }
    ids = [];
    for (i = 0, len = strings.length; i < len; i++) {
      s = strings[i];
      id = exports.stringToObjectIdSilently(s);
      if (id != null) {
        ids.push(id);
      }
    }
    return ids;
  };

  exports.toMongoCriteria = function(criteria) {
    var __type, mongoCriteria;
    if (!criteria) {
      return {};
    }
    __type = criteria.__type;
    delete criteria.__type;
    switch (__type) {
      case 'mongo':
        return criteria;
      case 'relation':
        mongoCriteria = {};
        toMongoCriteria(criteria, mongoCriteria);
        return mongoCriteria;
      default:
        return criteria;
    }
  };

  toMongoCriteria = function(criteria, mongoCriteria) {
    var fc, field, i, item, items, j, len, len1, mc, operator, ref, ref1, results, value;
    if (criteria == null) {
      return;
    }
    if (criteria.relation === 'or') {
      items = [];
      ref = criteria.items;
      for (i = 0, len = ref.length; i < len; i++) {
        item = ref[i];
        mc = {};
        toMongoCriteria(item, mc);
        if (mc) {
          items.push(mc);
        }
      }
      return mongoCriteria['$or'] = items;
    } else if (criteria.relation === 'and') {
      ref1 = criteria.items;
      results = [];
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        item = ref1[j];
        results.push(toMongoCriteria(item, mongoCriteria));
      }
      return results;
    } else if (criteria.field) {
      operator = criteria.operator;
      value = criteria.value;
      field = criteria.field;
      fc = mongoCriteria[field] = mongoCriteria[field] || {};
      switch (operator) {
        case '==':
          return mongoCriteria[field] = value;
        case '!=':
          return fc.$ne = value;
        case '>':
          return fc.$gt = value;
        case '>=':
          return fc.$gte = value;
        case '<':
          return fc.$lt = value;
        case '<=':
          return fc.$lte = value;
        case 'in':
          return fc.$in = value;
        case 'nin':
          return fc.$nin = value;
        case 'start':
          return fc.$regex = "^" + value;
        case 'end':
          return fc.$regex = value + "$";
        case 'contain':
          return fc.$regex = value;
      }
    }
  };

}).call(this);
