(function() {
  var MysqlConnection, MysqlStore, PoolConnection, Promise, _, _listCriteriaFields, arrayToSelectClause, buildInClause, config, criteriaToWhereClause, listCriteriaFields, log, mysql, objectToOrderClause, objectToSetClause, objectToWhereClause, relationCriteriaToWhereClause;

  Promise = require('bluebird');

  _ = require('lodash');

  mysql = require('mysql');

  PoolConnection = require('mysql/lib/PoolConnection');

  log = require('../log');

  config = require('../config');

  MysqlStore = (function() {
    function MysqlStore(config1) {
      this.config = config1;
    }

    MysqlStore.prototype._init = function() {
      this.pool = mysql.createPool({
        connectionLimit: 3,
        host: this.config.host,
        user: this.config.user,
        password: this.config.password,
        database: this.config.database
      });
      return this.pConnection = Promise.promisify(pool.getConnection.bind(pool));
    };

    MysqlStore.prototype.gDispose = function*() {
      var pEndPool;
      if (this.pool == null) {
        return;
      }
      log.system.info("Closing mysql [" + this.config.host + "]...");
      pEndPool = Promise.promisify(this.pool.end.bind(this.pool));
      try {
        return (yield pEndPool());
      } catch (error) {
        return log.system.error(e, 'dispose mysql', this.config.host(+"/" + this.config.database));
      }
    };

    MysqlStore.prototype.gConnect = function*() {
      var connection;
      if (this.pool == null) {
        this._init();
      }
      connection = (yield this.pConnection());
      return new MysqlConnection(connection);
    };

    MysqlStore.prototype.gWithTransaction = function*(gWork) {
      var conn, e, e2, r;
      conn = (yield* this.gConnect());
      try {
        yield conn.pBeginTransaction();
        r = (yield* gWork(conn));
        yield conn.pCommit();
        return r;
      } catch (error) {
        e = error;
        try {
          yield conn.pRollback();
        } catch (error) {
          e2 = error;
          log.system.error(e2, "autoCommit, rollback");
        }
        throw e;
      } finally {
        conn.release();
      }
    };

    MysqlStore.prototype.gWithoutTransaction = function*(gWork) {
      var conn;
      conn = (yield* this.gConnect());
      try {
        return (yield* gWork(conn));
      } finally {
        conn.release();
      }
    };

    MysqlStore.prototype.gReuseTransaction = function*(ctx, gWork) {
      if (ctx.conn == null) {
        yield* this.gWithTransaction(function(conn) {
          return ctx.conn = conn;
        });
      }
      return (yield* gWork(ctx.conn));
    };

    return MysqlStore;

  })();

  MysqlConnection = (function() {
    function MysqlConnection(conn1) {
      this.conn = conn1;
      this.pQuery = Promise.promisify(this.conn.query.bind(this.conn));
      this.pBeginTransaction = Promise.promisify(this.conn.beginTransaction.bind(this.conn));
      this.pCommit = Promise.promisify(this.conn.commit.bind(this.conn));
      this.pRollback = Promise.promisify(this.conn.rollback.bind(this.conn));
      this.pRead = this.pQuery;
    }

    MysqlConnection.prototype.release = function() {
      return this.conn.release();
    };

    MysqlConnection.prototype.pWrite = function() {
      this.written = true;
      return this.pQuery.apply(this.conn, arguments);
    };

    MysqlConnection.prototype.gFind = function*(arg) {
      var _w, criteria, criteriaFields, includedFields, list, orderBy, pageNo, pageSize, paging, r, select, skipLimit, sort, sql, table, values, where;
      table = arg.table, criteria = arg.criteria, includedFields = arg.includedFields, sort = arg.sort, pageNo = arg.pageNo, pageSize = arg.pageSize, paging = arg.paging;
      values = [];
      select = '*';
      where = '';
      orderBy = '';
      skipLimit = '';
      if (includedFields) {
        criteriaFields = listCriteriaFields(criteria);
        includedFields = _.uniqu(_.concat(includedFields, criteriaFields));
        select = arrayToSelectClause(includedFields);
      }
      if (criteria && _.size(criteria)) {
        _w = criteriaToWhereClause(criteria, values);
        if (_w) {
          where = "WHERE " + _w;
        }
      }
      if (sort) {
        orderBy = "ORDER BY " + objectToOrderClause(sort);
      }
      if (pageSize > 0) {
        if (pageNo < 1) {
          pageNo = 1;
        }
        skipLimit = "SKIP " + ((pageNo - 1) * pageSize) + " LIMIT " + pageSize;
      }
      sql = "select " + select + " from " + table + " " + where + " " + orderBy + " " + skipLimit;
      list = (yield this.pRead(sql, values));
      if (!paging) {
        return list;
      } else {
        sql = "select COUNT(1) as count from " + table + " " + where;
        r = (yield this.pRead(sql, values));
        return {
          total: r[0].count,
          page: list
        };
      }
    };

    MysqlConnection.prototype.gListByIds = function*(table, ids) {
      var inClause, sql, sqlValues;
      if (!(ids != null ? ids.length : void 0)) {
        return [];
      }
      sqlValues = [];
      inClause = buildInClause(ids, sqlValues);
      if (!inClause) {
        return [];
      }
      sql = "select * from " + table + " where _id IN " + inClause;
      return (yield this.pRead(sql, sqlValues));
    };

    MysqlConnection.prototype.gInsertOne = function*(table, object, keys) {
      return (yield* this.gInsertMany(table, [object], keys));
    };

    MysqlConnection.prototype.gInsertMany = function*(table, objects, keys) {
      var columns, j, k, key, len, len1, object, placeholders, placeholders2, sql, sqlValues;
      if (!(objects != null ? objects.length : void 0)) {
        return null;
      }
      if (!(keys != null ? keys.length : void 0)) {
        keys = _.keys(objects[0]);
      }
      if (!(keys != null ? keys.length : void 0)) {
        return null;
      }
      columns = (function() {
        var j, len, results;
        results = [];
        for (j = 0, len = keys.length; j < len; j++) {
          key = keys[j];
          results.push(mysql.escapeId(key));
        }
        return results;
      })();
      placeholders = [];
      sqlValues = [];
      for (j = 0, len = objects.length; j < len; j++) {
        object = objects[j];
        placeholders2 = [];
        for (k = 0, len1 = keys.length; k < len1; k++) {
          key = keys[k];
          placeholders2.push('?');
          sqlValues.push(object[key]);
        }
        placeholders.push("(" + (placeholders2.join(',')) + ")");
      }
      sql = "insert into " + table + "(" + (columns.join(',')) + ") values " + (placeholders.join(','));
      log.debug("sql,values", sql, sqlValues);
      return (yield this.pWrite(sql, sqlValues));
    };

    MysqlConnection.prototype.gUpdateByCriteria = function*(table, criteria, patch) {
      var set, sql, sqlValues, where;
      if (!_.size(criteria)) {
        return null;
      }
      if (!_.size(patch)) {
        return null;
      }
      sqlValues = [];
      set = objectToSetClause(patch, sqlValues);
      where = criteriaToWhereClause(criteria, sqlValues);
      sql = "update " + table + " set " + set + " where " + where;
      log.debug("sql,values", sql, sqlValues);
      return (yield this.pWrite(sql, sqlValues));
    };

    MysqlConnection.prototype.gDeleteManyByIds = function*(table, ids) {
      var inClause, sql, sqlValues;
      if (!(ids != null ? ids.length : void 0)) {
        return;
      }
      sqlValues = [];
      inClause = buildInClause(ids, sqlValues);
      sql = "delete * from " + table + " where _id IN " + inClause;
      return (yield this.pWrite(sql, sqlValues));
    };

    MysqlConnection.prototype.gDeleteManyByCriteria = function*(table, criteria) {
      var sql, sqlValues, where;
      if (!(typeof ids !== "undefined" && ids !== null ? ids.length : void 0)) {
        return;
      }
      sqlValues = [];
      where = criteriaToWhereClause(criteria, sqlValues);
      sql = "delete * from " + table + " where " + where;
      return (yield this.pWrite(sql, sqlValues));
    };

    return MysqlConnection;

  })();

  exports.init = function() {
    var mysqlConfig;
    mysqlConfig = config.mysql;
    if (mysqlConfig) {
      return exports.mysql = new MysqlStore(mysqlConfig);
    }
  };

  exports.gDispose = function*() {
    if (exports.mysql) {
      return (yield* exports.mysql.gDispose());
    }
  };

  exports.isIndexConflictError = function(e) {
    return e.code === 'ER_DUP_ENTRY';
  };

  arrayToSelectClause = function(array) {
    var f, fields;
    fields = (function() {
      var j, len, results;
      results = [];
      for (j = 0, len = array.length; j < len; j++) {
        f = array[j];
        results.push(mysql.escapeId(f));
      }
      return results;
    })();
    return fields.join(',');
  };

  criteriaToWhereClause = function(criteria, sqlValues) {
    var __type;
    __type = criteria.__type;
    delete criteria.__type;
    switch (__type) {
      case 'mongo':
        throw new Error('Cannot use mongo criteria for mysql');
        break;
      case 'relation':
        return relationCriteriaToWhereClause(criteria, sqlValues);
      default:
        return objectToWhereClause(criteria, sqlValues);
    }
  };

  objectToWhereClause = function(object, values) {
    var conditions, key, value;
    conditions = [];
    for (key in object) {
      value = object[key];
      conditions.push((mysql.escapeId(key)) + " = ?");
      values.push(value);
    }
    return conditions.join(' AND ');
  };

  relationCriteriaToWhereClause = function(criteria, sqlValues) {
    var comparedValue, field, i, item, items, j, len, operator, ref;
    if (criteria.relation) {
      items = [];
      ref = criteria.items;
      for (j = 0, len = ref.length; j < len; j++) {
        item = ref[j];
        i = relationCriteriaToWhereClause(item, sqlValues);
        if (i) {
          items.push(i);
        }
      }
      if (!items.length) {
        return null;
      }
      if (criteria.relation === 'or') {
        return "(" + items.join(" OR ") + ")";
      } else {
        return items.join(" AND ");
      }
    } else if (criteria.field) {
      operator = criteria.operator;
      comparedValue = criteria.value;
      field = mysql.escapeId(criteria.field);
      switch (operator) {
        case '==':
          sqlValues.push(comparedValue);
          return field + " = ?";
        case '!=':
          sqlValues.push(comparedValue);
          return field + " <> ?";
        case '>':
          sqlValues.push(comparedValue);
          return field + " > ?";
        case '>=':
          sqlValues.push(comparedValue);
          return field + " >= ?";
        case '<':
          sqlValues.push(comparedValue);
          return field + " < ?";
        case '<=':
          sqlValues.push(comparedValue);
          return field + " <= ?";
        case 'in':
          return field + " IN " + buildInClause(comparedValue, sqlValues);
        case 'nin':
          return field + " NOT IN " + buildInClause(comparedValue, sqlValues);
        case 'start':
          sqlValues.push(comparedValue + "%");
          return field + " LIKE ?";
        case 'end':
          sqlValues.push("%" + comparedValue);
          return field + " LIKE ?";
        case 'contain':
          sqlValues.push("%" + comparedValue + "%");
          return field + " LIKE ?";
        default:
          return null;
      }
    }
  };

  buildInClause = function(inList, sqlValues) {
    var i, j, len, placeholders;
    if (!(inList != null ? inList.length : void 0)) {
      return "";
    }
    placeholders = [];
    for (j = 0, len = inList.length; j < len; j++) {
      i = inList[j];
      placeholders.push("?");
      sqlValues.push(i);
    }
    return "(" + placeholders.join(',') + ")";
  };

  objectToOrderClause = function(object) {
    var key, orders, value;
    orders = [];
    for (key in object) {
      value = object[key];
      orders.push(mysql.escapeId(key) + " " + (value < 0 && "DESC" || "ASC"));
    }
    return orders.join(',');
  };

  objectToSetClause = function(object, values) {
    var key, set, value;
    set = [];
    for (key in object) {
      value = object[key];
      set.push((mysql.escapeId(key)) + " = ?");
      values.push(value);
    }
    return set.join(',');
  };

  _listCriteriaFields = function(criteria, list) {
    var item, j, len, ref, results;
    if (criteria.relation) {
      ref = criteria.items;
      results = [];
      for (j = 0, len = ref.length; j < len; j++) {
        item = ref[j];
        results.push(_listCriteriaFields(criteria, list));
      }
      return results;
    } else if (criteria.field) {
      return list.push(criteria.field);
    }
  };

  listCriteriaFields = function(criteria) {
    var list;
    list = [];
    _listCriteriaFields(criteria, list);
    return _.uniq(list);
  };

}).call(this);
