(function() {
  var Meta, gAddColumn, gColumnExists, gCreateTable, gDropTable, gRenameTable, gTableExists, log;

  Meta = require('../Meta');

  log = require('../log');

  exports.gSyncSchema = function*(mysql) {
    var entities, entityMeta, entityName, results;
    entities = Meta.getEntities();
    results = [];
    for (entityName in entities) {
      entityMeta = entities[entityName];
      if (entityMeta.db !== Meta.DB.mysql) {
        continue;
      }
      results.push((yield* mysql.gWithoutTransaction(function*(conn) {
        var fieldMeta, fieldName, ref, results1;
        if (!(yield* gTableExists(entityMeta.tableName, conn))) {
          log.system.info('create table for ' + entityName);
          return (yield* gCreateTable(entityMeta, conn));
        } else {
          ref = entityMeta.fields;
          results1 = [];
          for (fieldName in ref) {
            fieldMeta = ref[fieldName];
            if (!(yield* gColumnExists(entityMeta.tableName, fieldName, conn))) {
              log.system.info('add column ' + fieldName + ' for ' + entityName);
              results1.push((yield* gAddColumn(entityMeta, fieldMeta, conn)));
            } else {
              results1.push(void 0);
            }
          }
          return results1;
        }
      })));
    }
    return results;
  };

  gCreateTable = function*(entityMeta, conn) {
    var args, columns, fieldMeta, fieldName, notNull, ref, sql, sqlType;
    sql = "CREATE TABLE ?? (";
    columns = [];
    args = [entityMeta.tableName];
    ref = entityMeta.fields;
    for (fieldName in ref) {
      fieldMeta = ref[fieldName];
      sqlType = fieldMeta.sqlColM ? fieldMeta.persistType + "(" + fieldMeta.sqlColM + ")" : fieldMeta.persistType;
      notNull = fieldMeta.required && "NOT NULL" || "NULL";
      columns.push("?? " + sqlType + " " + notNull);
      args.push(fieldName);
    }
    sql = sql + columns.join(", ") + ", PRIMARY KEY (`_id`))";
    return (yield conn.pWrite(sql, args));
  };

  gRenameTable = function*(oldName, newName, conn) {
    return (yield conn.pWrite("alter table ?? rename ??", [oldName, newName]));
  };

  gDropTable = function*(name, conn) {
    return (yield conn.pWrite("drop table ??", [name]));
  };

  gAddColumn = function*(entityMeta, fieldMeta, conn) {
    var args, notNull, sql, sqlType;
    sql = "ALTER TABLE ?? Add Column ?? ";
    args = [entityMeta.tableName, fieldName];
    sqlType = fieldMeta.sqlColM ? fieldMeta.persistType + "(" + fieldMeta.sqlColM + ")" : fieldMeta.persistType;
    notNull = fieldMeta.required && "NOT NULL" || "";
    sql += sqlType + " " + notNull;
    return (yield conn.pWrite(sql, args));
  };

  gTableExists = function*(tableName, conn) {
    var r;
    r = (yield conn.pRead("SHOW TABLES LIKE ?", [tableName]));
    return r != null ? r.length : void 0;
  };

  gColumnExists = function*(tableName, columnName, conn) {
    var r;
    r = (yield conn.pRead("SHOW COLUMNS FROM ?? LIKE ?", [tableName, columnName]));
    return r != null ? r.length : void 0;
  };

}).call(this);
