(function() {
  var ObjectId, Promise, _, co, log, parseXMLString, xml2js, xmlBuilder;

  _ = require('lodash');

  log = require('./log');

  Promise = require('bluebird');

  co = require('co');

  xml2js = require("xml2js");

  ObjectId = require('mongodb').ObjectId;

  log = require('./log');

  exports.objectToKeyValuePairString = function(obj) {
    var k, v;
    return obj && _.join((function() {
      var results;
      results = [];
      for (k in obj) {
        v = obj[k];
        results.push(k + "=" + v);
      }
      return results;
    })(), "&") || "";
  };

  exports.stringToInt = function(string, alternative) {
    var value;
    if (_.isNumber(string)) {
      return string;
    }
    if (string == null) {
      if (alternative != null) {
        return alternative;
      } else {
        return string;
      }
    }
    if (!_.isString(string)) {
      if (alternative != null) {
        return alternative;
      } else {
        return void 0;
      }
    }
    string = exports.trimString(string);
    if (!string) {
      if (alternative != null) {
        return alternative;
      } else {
        return void 0;
      }
    }
    value = parseInt(string, 10);
    if (_.isNaN(value)) {
      if (alternative != null) {
        return alternative;
      } else {
        return value;
      }
    }
    return value;
  };

  exports.trimString = function(string) {
    if (string == null) {
      return string;
    }
    return string.replace(/(^\s*)|(\s*$)/g, "");
  };

  exports.stringToFloat = function(string, alternative) {
    var value;
    if (_.isNumber(string)) {
      return string;
    }
    if (string == null) {
      if (alternative != null) {
        return alternative;
      } else {
        return string;
      }
    }
    if (!_.isString(string)) {
      if (alternative != null) {
        return alternative;
      } else {
        return void 0;
      }
    }
    string = exports.trimString(string);
    if (!string) {
      if (alternative != null) {
        return alternative;
      } else {
        return void 0;
      }
    }
    value = parseFloat(string);
    if (_.isNaN(value)) {
      if (alternative != null) {
        return alternative;
      } else {
        return value;
      }
    }
    return value;
  };

  exports.longToDate = function(long) {
    if (_.isDate(long)) {
      return long;
    }
    if (long == null) {
      return long;
    }
    return new Date(long);
  };

  exports.dateToLong = function(date) {
    if (date == null) {
      return date;
    }
    return date.getTime();
  };

  exports.toBoolean = function(v) {
    return Boolean(v);
  };

  exports.stringToBoolean = function(value) {
    if (_.isBoolean(value)) {
      return value;
    } else if (value === "false") {
      return false;
    } else if (value === "true") {
      return true;
    } else if (_.isNull(value)) {
      return null;
    } else {
      return void 0;
    }
  };

  exports.arrayToTrueObject = function(array) {
    var a, j, len, o;
    if (!array) {
      return null;
    }
    o = {};
    for (j = 0, len = array.length; j < len; j++) {
      a = array[j];
      o[a] = true;
    }
    return o;
  };

  exports.splitString = function(string, s) {
    var a, a1, a2, i, j, len;
    string = _.trim(string);
    if (!string) {
      return null;
    }
    a1 = _.split(string, s);
    a2 = [];
    for (j = 0, len = a1.length; j < len; j++) {
      a = a1[j];
      i = _.trim(a);
      if (i) {
        a2.push(i);
      }
    }
    return a2;
  };

  exports.setIfNone = function(object, field, alt) {
    var v;
    v = object[field];
    if (v != null) {
      return v;
    }
    object[field] = _.isFunction(alt) ? alt() : alt;
    return alt;
  };

  exports.isGenerator = function(fn) {
    return (fn != null ? fn.constructor.name : void 0) === 'GeneratorFunction';
  };

  xmlBuilder = new xml2js.Builder({
    rootName: 'xml',
    headless: true
  });

  parseXMLString = Promise.promisify(xml2js.parseString.bind(xml2js));

  exports.objectToXML = function(object) {
    return xmlBuilder.buildObject(object);
  };

  exports.pParseXML = parseXMLString;

  exports.setTimeout = function(timeout, run) {
    return setTimeout(run, timeout);
  };

  exports.setGeneratorTimeout = function(timeout, run) {
    var run2;
    run2 = function() {
      return co(run)["catch"](function(e) {
        return log.system.error(e);
      });
    };
    return setTimeout(run2, timeout);
  };

  exports.setGeneratorInterval = function(timeout, run) {
    var run2;
    run2 = function() {
      return co(run)["catch"](function(e) {
        return log.system.error(e);
      });
    };
    return setInterval(run2, timeout);
  };

  exports.entityListToIdMap = function(list) {
    var i, j, len, map;
    map = {};
    for (j = 0, len = list.length; j < len; j++) {
      i = list[j];
      map[i._id] = i;
    }
    return map;
  };

  exports.objectIdsEquals = function(a, b) {
    return (a != null ? a.toString() : void 0) === (b != null ? b.toString() : void 0);
  };

  exports.inObjectIds = function(targetId, ids) {
    var id, j, len;
    for (j = 0, len = ids.length; j < len; j++) {
      id = ids[j];
      if ((id != null ? id.toString() : void 0) === targetId) {
        return true;
      }
    }
    return false;
  };

  exports.addEqualsConditionToListCriteria = function(query, field, value) {
    var criteria, item, obj1;
    if (query.criteria) {
      criteria = query.criteria;
      item = {
        field: field,
        value: value,
        operator: '=='
      };
      if (criteria.type === 'relation') {
        if (criteria.relation === 'and') {
          return query.criteria.items.push(item);
        } else if (criteria.relation === 'or') {
          return query.criteria = {
            __type: 'relation',
            relation: 'and',
            items: [criteria, item]
          };
        } else {
          return query.criteria = {
            __type: 'relation',
            relation: 'and',
            items: [criteria, item]
          };
        }
      }
    } else {
      return query.criteria = (
        obj1 = {},
        obj1["" + field] = value,
        obj1
      );
    }
  };

  exports.jsObjectToTypedJSON = function(jsObject) {
    var addType, j, jsonObject, key, len, results, value;
    if (jsObject == null) {
      return jsObject;
    }
    addType = function(value) {
      if (_.isDate(value)) {
        return {
          _type: 'Date',
          _value: value.getTime()
        };
      } else if (value instanceof ObjectId) {
        return {
          _type: 'ObjectId',
          _value: value.toString()
        };
      } else if (_.isObject(value)) {
        return {
          _type: 'json',
          _value: exports.jsObjectToTypedJSON(value)
        };
      } else {
        return {
          _type: '',
          _value: value
        };
      }
    };
    if (_.isArray(jsObject)) {
      results = [];
      for (j = 0, len = jsObject.length; j < len; j++) {
        value = jsObject[j];
        results.push(addType(value));
      }
      return results;
    } else if (_.isObject(jsObject)) {
      jsonObject = {};
      for (key in jsObject) {
        value = jsObject[key];
        jsonObject[key] = addType(value);
      }
      return jsonObject;
    } else {
      return jsObject;
    }
  };

  exports.typedJSONToJsObject = function(jsonObject) {
    var j, jsObject, key, len, removeType, results, value;
    if (jsonObject == null) {
      return jsonObject;
    }
    removeType = function(value) {
      switch (value._type) {
        case 'Date':
          return new Date(value._value);
        case 'ObjectId':
          return new ObjectId(value._value);
        case 'json':
          return exports.typedJSONToJsObject(value._value);
        default:
          return value._value;
      }
    };
    if (_.isArray(jsonObject)) {
      results = [];
      for (j = 0, len = jsonObject.length; j < len; j++) {
        value = jsonObject[j];
        results.push(removeType(value));
      }
      return results;
    } else if (_.isObject(jsonObject)) {
      jsObject = {};
      for (key in jsonObject) {
        value = jsonObject[key];
        jsObject[key] = removeType(value);
      }
      return jsObject;
    } else {
      return jsonObject;
    }
  };

}).call(this);
