(function() {
  var Promise, config, error, extension, gCatchError, http, log, server;

  Promise = require('bluebird');

  http = require('http');

  log = require('../log');

  error = require('../error');

  config = require('../config');

  extension = require('../extension');

  server = null;

  gCatchError = function*(next) {
    var e, errorCatcher, len, ref, ref1, url;
    errorCatcher = (ref = this.route.info) != null ? ref.errorCatcher : void 0;
    if (errorCatcher) {
      return (yield errorCatcher.call(this, next));
    } else {
      try {
        return (yield next);
      } catch (error1) {
        e = error1;
        if (e instanceof error.Error401) {
          this.status = 401;
          if ((ref1 = this.route.info) != null ? ref1.isPage : void 0) {
            len = config.urlPrefix.length + 1;
            url = this.request.originalUrl.substring(len);
            return this.redirect('sign-in?callback=' + encodeURIComponent(url));
          } else {
            return this.body = e.describe();
          }
        } else if (e instanceof error.Error403) {
          this.status = 403;
          return this.body = e.describe();
        } else if (e instanceof error.UserError) {
          this.status = 400;
          return this.body = e.describe();
        } else {
          this.status = 500;
          return log.system.error(e, e.message, 'catch all');
        }
      }
    }
  };

  exports.gStart = function*() {
    var ac, enableDestroy, formidableConfig, koa, koaBody, koaServer, router;
    koa = require('koa');
    koaServer = koa();
    koaServer.keys = [config.cookieKey];
    koaServer.proxy = true;
    koaServer.use(require('./jade').jade.middleware);
    router = require('./router');
    router.refresh();
    koaServer.use(router.parseRoute);
    koaServer.use(gCatchError);
    ac = require('../handler/AccessController');
    koaServer.use(ac.gIdentifyUser);
    koaServer.use(ac.gControlAccess);
    koaBody = require('koa-body');
    formidableConfig = {
      uploadDir: config.uploadPath,
      keepExtensions: true,
      maxFieldsSize: config.httpBodyMaxFieldsSize
    };
    koaServer.use(koaBody({
      multipart: true,
      formidable: formidableConfig
    }));
    if (extension.koaMiddlewareBeforeHandler) {
      koaServer.use(extension.koaMiddlewareBeforeHandler);
    }
    koaServer.use(router.handleRoute);
    server = http.createServer(koaServer.callback());
    server.on('error', function(err) {
      return log.system.error(err, 'Error on server!');
    });
    enableDestroy = require('server-destroy');
    enableDestroy(server);
    return (yield Promise.promisify(server.listen.bind(server))(config.serverPort));
  };

  exports.stop = function(onWebServerClosed) {
    if (server) {
      server.on('close', function() {
        return onWebServerClosed();
      });
      return server.destroy();
    } else {
      return onWebServerClosed();
    }
  };

}).call(this);
