(function() {
  var Meta, actions, k, router, v;

  router = require('./router');

  Meta = require('../Meta');

  actions = {
    ReadMeta: '读取元数据',
    WriteMeta: '修改元数据',
    ChangePhone: '修改绑定手机接口',
    ChangeEmail: '修改绑定邮箱接口',
    PreFilters: '管理预定义查询',
    Upload: '文件上传',
    RichTextUpload: '富文本编辑器文件上传',
    ES: '配置搜索引擎',
    Promotion: '推广活动',
    ViewCache: '查看缓存',
    ClearEntityCache: '清除实体缓存'
  };

  for (k in actions) {
    v = actions[k];
    Meta.actions[k] = v;
  }

  exports.addCommonRouteRules = function(rrr) {
    var CaptchaHandler, ElasticSearchController, EntityHandler, MetaHandler, SecurityCodeHandler, UploadHandler, UserHandler;
    MetaHandler = require('../handler/MetaHandler');
    rrr.get('/meta', {
      action: 'ReadMeta'
    }, MetaHandler.gGetAllMeta);
    rrr.get('/meta-empty', {
      action: 'WriteMeta'
    }, MetaHandler.gGetEmptyEntityMeta);
    rrr.get('/meta/:type/:name', {
      action: 'ReadMeta'
    }, MetaHandler.gGetMeta);
    rrr.put('/meta/:type/:name', {
      action: 'WriteMeta'
    }, MetaHandler.gSaveMeta);
    rrr.post('/meta', {
      action: 'WriteMeta'
    }, MetaHandler.gImportMeta);
    rrr.del('/meta/:type/:name', {
      action: 'WriteMeta'
    }, MetaHandler.gRemoveMeta);
    rrr.get('/meta/actions', {
      action: 'WriteMeta'
    }, MetaHandler.gGetActions);
    UserHandler = require('../handler/UserHandler');
    rrr.get('/api/ping', {
      auth: true
    }, UserHandler.gPing);
    rrr.post('/api/sign-in', {}, UserHandler.gSignIn);
    rrr.post('/api/sign-out', {
      auth: true
    }, UserHandler.gSignOut);
    rrr.post('/api/change-password', {
      auth: true
    }, UserHandler.gChangePassword);
    rrr.post('/api/reset-password', {}, UserHandler.gResetPassword);
    rrr.post('/api/change-phone', {
      action: 'ChangePhone'
    }, UserHandler.gChangePhone);
    rrr.post('/api/change-email', {
      action: 'ChangeEmail'
    }, UserHandler.gChangeEmail);
    SecurityCodeHandler = require('../handler/SecurityCodeHandler');
    rrr.post('/api/security-code/phone/:phone', {}, SecurityCodeHandler.gSendSignUpCodeToPhone);
    rrr.post('/api/security-code/email/:email', {}, SecurityCodeHandler.gSendSignUpCodeToEmail);
    CaptchaHandler = require('../handler/CaptchaHandler');
    rrr.get('/captcha', {}, CaptchaHandler.gGenerate);
    EntityHandler = require('../handler/EntityHandler');
    rrr.get('/entity/:entityName', {
      auth: 'listEntity'
    }, EntityHandler.gList);
    rrr.get('/entity/:entityName/:id', {
      auth: 'getEntity'
    }, EntityHandler.gFindOneById);
    rrr.post('/entity/:entityName', {
      auth: 'createEntity'
    }, EntityHandler.gCreateEntity);
    rrr.put('/entity/:entityName/:id', {
      auth: 'updateOneEntity'
    }, EntityHandler.gUpdateEntityById);
    rrr.put('/entity/:entityName', {
      auth: 'updateManyEntity'
    }, EntityHandler.gUpdateEntityInBatch);
    rrr.del('/entity/:entityName', {
      auth: 'removeEntity'
    }, EntityHandler.gDeleteEntityInBatch);
    rrr.post('/entity/:entityName/recover', {
      auth: 'recoverEntity'
    }, EntityHandler.gRecoverInBatch);
    rrr.put('/entity/filters', {
      action: 'PreFilters'
    }, EntityHandler.gSaveFilters);
    rrr.del('/entity/filters', {
      action: 'PreFilters'
    }, EntityHandler.gRemoveFilters);
    rrr.get('/cache/memory', {
      action: 'ViewCache'
    }, EntityHandler.gViewMemoryCache);
    rrr.del('/cache', {
      action: 'ClearEntityCache'
    }, EntityHandler.gClearAllCache);
    UploadHandler = require('../handler/UploadHandler');
    rrr.post('/file', {
      action: 'Upload'
    }, UploadHandler.gUpload);
    rrr.post('/file2', {
      action: 'Upload'
    }, UploadHandler.gUpload2);
    rrr.post('/rich-text-file', {
      action: 'RichTextUpload'
    }, UploadHandler.gUploadForRichText);
    ElasticSearchController = require('../handler/ElasticSearchController');
    return rrr.post('/config-es', {
      action: 'ES'
    }, ElasticSearchController.gConfig);
  };

}).call(this);
