(function() {
  var RouteRuleRegisters, _, addRouteRules, collectRouteUrls, compose, log, mapping, match, rootMapping, routes, splitPath, test, urlLib, util,
    slice = [].slice;

  compose = require('koa-compose');

  urlLib = require('url');

  _ = require('lodash');

  log = require('../log');

  util = require('../util');

  routes = {};

  rootMapping = {};

  mapping = {};

  splitPath = function(aPath) {
    var parts, partsEnd, partsStart;
    parts = aPath.split("/");
    partsStart = parts[0] ? 0 : 1;
    partsEnd = parts[parts.length - 1] ? parts.length : parts.length - 1;
    return parts.slice(partsStart, partsEnd);
  };

  addRouteRules = function() {
    var h, handler, handlers, i, info, key, len, method, route, url;
    method = arguments[0], url = arguments[1], info = arguments[2], handlers = 4 <= arguments.length ? slice.call(arguments, 3) : [];
    key = method + url;
    for (i = 0, len = handlers.length; i < len; i++) {
      h = handlers[i];
      if (!util.isGenerator(h)) {
        throw new Error(method + " " + url + ", " + h + " is not a generator");
      }
    }
    handler = handlers.length === 1 ? handlers[0] : compose(handlers);
    route = {
      method: method,
      url: url,
      info: info,
      handler: handler,
      indexToVariable: {}
    };
    return routes[key] = route;
  };

  RouteRuleRegisters = (function() {
    function RouteRuleRegisters(urlPrefix, errorCatcher) {
      this.errorCatcher = errorCatcher;
      if (!urlPrefix) {
        throw new Error('urlPrefix cannot be empty');
      }
      if (urlPrefix[urlPrefix.length - 1] === '/') {
        urlPrefix = urlPrefix.substring(0, urlPrefix.length - 1);
      }
      this.urlPrefix = urlPrefix;
    }

    RouteRuleRegisters.prototype.add = function() {
      var handlers, info, method, url;
      method = arguments[0], url = arguments[1], info = arguments[2], handlers = 4 <= arguments.length ? slice.call(arguments, 3) : [];
      url = url === '' || url === '/' ? '' : url[0] === '/' ? url.substring(1) : void 0;
      url = this.urlPrefix + "/" + url;
      info = info || {};
      info.errorCatcher = this.errorCatcher;
      info.urlPrefix = this.urlPrefix;
      return addRouteRules.apply(null, [method, url, info].concat(slice.call(handlers)));
    };

    RouteRuleRegisters.prototype.get = function() {
      var handlers, info, url;
      url = arguments[0], info = arguments[1], handlers = 3 <= arguments.length ? slice.call(arguments, 2) : [];
      return this.add.apply(this, ['get', url, info].concat(slice.call(handlers)));
    };

    RouteRuleRegisters.prototype.post = function() {
      var handlers, info, url;
      url = arguments[0], info = arguments[1], handlers = 3 <= arguments.length ? slice.call(arguments, 2) : [];
      return this.add.apply(this, ['post', url, info].concat(slice.call(handlers)));
    };

    RouteRuleRegisters.prototype.put = function() {
      var handlers, info, url;
      url = arguments[0], info = arguments[1], handlers = 3 <= arguments.length ? slice.call(arguments, 2) : [];
      return this.add.apply(this, ['put', url, info].concat(slice.call(handlers)));
    };

    RouteRuleRegisters.prototype.del = function() {
      var handlers, info, url;
      url = arguments[0], info = arguments[1], handlers = 3 <= arguments.length ? slice.call(arguments, 2) : [];
      return this.add.apply(this, ['delete', url, info].concat(slice.call(handlers)));
    };

    return RouteRuleRegisters;

  })();

  exports.RouteRuleRegisters = RouteRuleRegisters;

  exports.refresh = function() {
    var i, index, key, len, mOfIndex, mOfLength, mOfMethod, mOfTerm, method, name, part, parts, partsLength, route, routeWeight, url;
    rootMapping = {};
    mapping = {};
    for (key in routes) {
      route = routes[key];
      url = route.url;
      method = route.method;
      route.indexToVariable = {};
      if (url === '' || url === '/') {
        rootMapping[method] = url;
      } else {
        parts = splitPath(url);
        partsLength = parts.length;
        mOfMethod = util.setIfNone(mapping, method, {});
        mOfLength = util.setIfNone(mOfMethod, partsLength, []);
        routeWeight = 0;
        for (index = i = 0, len = parts.length; i < len; index = ++i) {
          part = parts[index];
          mOfIndex = util.setIfNone(mOfLength, index, {
            terms: {},
            variable: []
          });
          if (part[0] === ':') {
            name = part.slice(1);
            route.indexToVariable[name] = index;
            mOfIndex.variable.push(url);
          } else {
            mOfTerm = util.setIfNone(mOfIndex.terms, part, []);
            mOfTerm.push(url);
            routeWeight = routeWeight + (1 << (partsLength - index - 1));
          }
        }
        route.routeWeight = routeWeight;
      }
    }
    return log.system.info('routes: ' + _.size(routes));
  };

  collectRouteUrls = function(mOfIndex, part) {
    var i, j, len, len1, routeUrlMap, routeUrls, u;
    routeUrlMap = {};
    routeUrls = mOfIndex.terms[part];
    if (routeUrls) {
      for (i = 0, len = routeUrls.length; i < len; i++) {
        u = routeUrls[i];
        routeUrlMap[u] = true;
      }
    }
    routeUrls = mOfIndex.variable;
    if (routeUrls) {
      for (j = 0, len1 = routeUrls.length; j < len1; j++) {
        u = routeUrls[j];
        routeUrlMap[u] = true;
      }
    }
    return routeUrlMap;
  };

  match = function(method, path, params) {
    var finalRoute, i, index, len, mOfIndex, mOfLength, maxRouteWeight, name, newPossibleRouteUrl, part, parts, possibleRouteUrl, ref, ref1, route, routeUrl, u, v;
    method = method.toLowerCase();
    parts = splitPath(path);
    if (path === '' || path === '/') {
      routeUrl = rootMapping[method];
      if (!routeUrl) {
        return null;
      }
      return routes[method + routeUrl];
    } else {
      mOfLength = (ref = mapping[method]) != null ? ref[parts.length] : void 0;
      if (!mOfLength) {
        return null;
      }
      possibleRouteUrl = {};
      for (index = i = 0, len = parts.length; i < len; index = ++i) {
        part = parts[index];
        mOfIndex = mOfLength[index];
        if (!mOfIndex) {
          return null;
        }
        if (index === 0) {
          possibleRouteUrl = collectRouteUrls(mOfIndex, part);
        } else {
          newPossibleRouteUrl = collectRouteUrls(mOfIndex, part);
          for (u in possibleRouteUrl) {
            v = possibleRouteUrl[u];
            if (!newPossibleRouteUrl[u]) {
              delete possibleRouteUrl[u];
            }
          }
        }
        if (!_.size(possibleRouteUrl)) {
          return null;
        }
      }
      maxRouteWeight = 0;
      finalRoute = null;
      for (routeUrl in possibleRouteUrl) {
        v = possibleRouteUrl[routeUrl];
        route = routes[method + routeUrl];
        if (route.routeWeight > maxRouteWeight) {
          finalRoute = route;
          maxRouteWeight = route.routeWeight;
        }
      }
      ref1 = finalRoute.indexToVariable;
      for (name in ref1) {
        index = ref1[name];
        params[name] = parts[index];
      }
      return finalRoute;
    }
  };

  exports.parseRoute = function*(next) {
    var params, path, route;
    path = decodeURI(this.request.path);
    params = {};
    route = match(this.request.method, path, params);
    if (route) {
      this.params = params;
      this.route = route;
      return (yield next);
    } else {
      log.debug('fail to match route,', {
        method: this.request.method,
        path: path
      });
      return this.status = 404;
    }
  };

  exports.handleRoute = function*(next) {
    return (yield this.route.handler.call(this, next));
  };

  test = function() {
    var params;
    exports.addRouteRules('get', "/", {
      action: "index"
    }, function(next) {
      return true;
    });
    exports.addRouteRules('get', "/home", {
      action: "home"
    }, function(next) {
      return true;
    });
    exports.addRouteRules('get', "/meta", {
      action: "meta"
    }, function(next) {
      return true;
    });
    exports.addRouteRules('post', "/meta", {
      action: "meta"
    }, function(next) {
      return true;
    });
    exports.addRouteRules('put', "/meta/:name", {
      action: "meta"
    }, function(next) {
      return true;
    });
    exports.addRouteRules('put', "/meta/_blank", {
      action: "meta"
    }, function(next) {
      return true;
    });
    exports.addRouteRules('put', "/meta/:name/fields", {
      action: "meta"
    }, function(next) {
      return true;
    });
    exports.addRouteRules('get', "/entity/:name/:id", {
      action: "entity"
    }, function(next) {
      return true;
    });
    exports.refresh();
    log.debug("pathTree", JSON.stringify(mapping, null, 4));
    params = {};
    return console.log(match('get', '/entity/User/1', params), params);
  };

}).call(this);
